/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/
 
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include "incl.h"

 Car *free_cars = NULL;
 
 extern double uniform(double,double,long *);
 extern void schedule(int, double, EventParam);

void in_list_car(Car **last, Car *elem)
{
  Car *p;
  if (elem==NULL) return;         
  
  /* empty Event List */
  if ((*last)==NULL)
    {
       (*last) = elem;
       (*last)->next = elem;
       (*last)->prev = elem;
       return;
    }
  /*REALISTIC TRAFFIC: sorted per position*/
  if(traffic.model > TRAFFIC_NBNA){
  /*  it is the last element*/
       if (elem->x <= (*last)->x){
	    elem->prev = (*last);
	    elem->next = (*last)->next;
	    ((*last)->next)->prev = elem;
	    (*last)->next = elem;
	    (*last) = elem;
       }else {
       /* 
       ** else I need to find the exact position (ordered list)
       */
	    p = (*last);
	    while (elem->x > p->x && p->prev!=(*last))
              p = p->prev;
	      if (elem->x > p->x){ 
        	elem->prev = p->prev;
        	elem->next = p;
        	(p->prev)->next = elem;
        	p->prev = elem;
              }else{
        	elem->prev = p;
        	elem->next = p->next;
        	(p->next)->prev = elem;
        	p->next = elem;
              }
	 }
  /*NBNA TRAFFIC: sorted per exitTime*/
  } else {
  /*  it is the last element*/
       if (elem->exitTime >= (*last)->exitTime){
	    elem->prev = (*last);
	    elem->next = (*last)->next;
	    ((*last)->next)->prev = elem;
	    (*last)->next = elem;
	    (*last) = elem;
       }else {
       /* 
       ** else I need to find the exact position (ordered list)
       */
	    p = (*last);
	    while (elem->exitTime < p->exitTime && p->prev!=(*last))
              p = p->prev;
	      if (elem->exitTime < p->exitTime){ 
        	elem->prev = p->prev;
        	elem->next = p;
        	(p->prev)->next = elem;
        	p->prev = elem;
              }else{
        	elem->prev = p;
        	elem->next = p->next;
        	(p->next)->prev = elem;
        	p->next = elem;
              }
	 }

  }
  return;
}


Car *out_list(Car **last)
{
  Car *p;
   // check_list(last);
  if ((*last)==NULL) return NULL;   /* L'Event List e' vuota */
  if ((*last)->next==(*last))
    {
	  /* Sto estraendo l'unico elemento della Event List */
      p = (*last);           
      (*last) = NULL;
    }
  else
    {
	  /* Estraggo il primo elemento (successore di last) */
      p = (*last)->next;
      (p->next)->prev = (*last);
      (*last)->next = p->next;
    }
   // check_list(last);
  return p;
}

extern int H_speed, H_rho;
void updatePosition(Car **last) 
{
  Car *p, *oldest;

  //DBG_I(UpdatePosition);
  //DBG_PL(UpdatePosition,"Time:%f  cars:%d"),current_time/ONE_SECOND,total_cars DBG_PR;
  
  if ((*last)==NULL)
       return;
  
  oldest = p = (*last)->next;
  
  if(traffic.model > TRAFFIC_NBNA){
      do {

	//DBG_PL(UpdatePosition,"carId: %d (first:%d last:%d)"), p->id,(*last)->next->id,(*last)->id DBG_PR;
	//DBG_PL(UpdatePosition,"carId: %d (pos:%f v:%f)"), p->id,p->x,p->v DBG_PR;


	if((p->prev)->x > p->x)
	   p->d=(p->prev)->x-p->x-1; 
	else
	   p->d=(p->prev)->x+STREET_LENGTH/traffic.cell-p->x-1; 
	
	if ((algo == ALGO_NONE) && (current_time/ONE_SECOND>10)) {
	    histo_add(H_distance, p->d*traffic.cell);
	    histo_add(H_headw,   (p->d*traffic.cell)/(p->v*(traffic.cell/traffic.step)));
	    histo_add(H_speed, p->v*(traffic.cell/traffic.step) * 3.6);
	}
	
	int pid;
	double v1, gap, vsafe, noise, oldspeed=p->v, pref;
	switch (traffic.model) {
   	    case TRAFFIC_NASCH:
	    case TRAFFIC_NASCH_MINISTEP:
 		p->v=min(p->v+traffic.vunit,traffic.vmax);
		//DBG_PL(UpdatePosition,"NASCH v-acc: %f "), p->v DBG_PR;

		//DBG_PL(UpdatePosition,"NASCH v-dec: %f"), p->v DBG_PR;
		p->v=min(p->v,max(0,p->d*traffic.vunit));    
		
		if ( uniform(0.,1.,&seed) <= traffic.pdec) {
  	              //DBG_PL(UpdatePosition,"RANDOM !!! v-dec: %f"), p->v DBG_PR; 
		       p->v=max(p->v-traffic.vunit, 0.0);
		}			
		break;	

	
    	    case TRAFFIC_VDR:
	    case TRAFFIC_VDR_MINISTEP:
	        // try to accelerate by one vunit		
 		p->v=min(p->v+traffic.vunit,traffic.vmax);

	        // respect collision-avoidance safety-gap
		p->v=min(p->v,max(0,p->d*traffic.vunit));    

		// velocity-dependant randomization	
		pid = (int) floor(oldspeed/traffic.vunit);
		// 		printf("%d %f %d\n",pid, traffic.vmax, (int)floor(oldspeed/traffic.vunit) );
		// 		assert( pid <= traffic.vmax );
		if ( uniform(0.,1.,&seed) <= traffic.vdr.pdec[pid]) 
	    	    p->v=max(p->v-traffic.vunit, 0.0);

		break;


    	    case TRAFFIC_TSQ:
	    case TRAFFIC_TSQ_MINISTEP:
	        // try to accelerate by one vunit
 		p->v=min(p->v+traffic.vunit,traffic.vmax);

	        // respect collision-avoidance safety-gap
		p->v=min(p->v,max(0,p->d*traffic.vunit));    


		// velocity-dependant randomization,
		// with extra-safe decrease when gap<=1 
		pid = (int) round(p->v/traffic.vunit);
		pref = traffic.vdr.pdec[pid] + 
			      (p->d/traffic.dunit<=1.0) ? 
			      		traffic.tsq.incr_pdec : 
					0.0;		
					
		if ( uniform(0.,1.,&seed) <= min(1.0, pref) ) 
	    	    p->v=max(p->v-traffic.vunit, 0.0);

		break;

				   
    	    case TRAFFIC_FUKUI:
	    case TRAFFIC_FUKUI_MINISTEP:
	        // try to accelerate by one vunit
 		p->v=min(p->v+traffic.vunit,traffic.vmax);

	        // respect collision-avoidance safety-gap
		p->v=min(p->v,max(0,p->d*traffic.vunit));    


		// speed-crazy japanese people probabilistically decelerate
		// provided that they are traveling at maximum speed
		if ( p->v == traffic.vmax ) {
		    if ( uniform(0.,1.,&seed) <= traffic.vdr.pdec[((int)traffic.vmax)]) 
	    	    	p->v=max(p->v-traffic.vunit, 0.0);
		}					  
		break;		


    	    case TRAFFIC_GIPPS:
	    case TRAFFIC_GIPPS_MINISTEP:
		   v1  = (p->prev->v);
		   gap = p->d/(traffic.dunit/traffic.vunit);
		   vsafe = v1 + (gap-v1*traffic.step)/((v1+p->v)/(2*traffic.gipps.b) + traffic.step);
    		   noise = uniform(0.0,traffic.gipps.a*traffic.gipps.epsilon,&seed);

		  //DBG_PL(UpdatePosition,"GIBBS v-acc: %f vasfe: %f "), p->v+traffic.gipps.a*traffic.step, vsafe DBG_PR;
		   p->v=min(p->v+traffic.gipps.a*traffic.step, vsafe);
		   p->v=min(p->v, traffic.vmax);
	
 		   p->v=max(p->v-noise,0.0);
		  //DBG_PL(UpdatePosition,"GIBBS v-dec: %f  "), p->v DBG_PR;
		   break;	

	    case TRAFFIC_TOCA:
		   gap = p->d/(traffic.dunit/traffic.vunit);
		   DBG_PL(UpdatePosition,"gap:%f v:%f v*1.1:%f"),gap , p->v,p->v*1.1 DBG_PR;
		   if(gap > p->v*1.1)
			if ( uniform(0.,1.,&seed) <= 0.9) 
		   		p->v=min(p->v+traffic.vunit, traffic.vmax);
		   DBG_PL(UpdatePosition,"v(after rand):%f "),p->v DBG_PR;
			
		   p->v = min(p->v, gap);
		   DBG_PL(UpdatePosition,"v(after gap-cmp):%f "),p->v DBG_PR;
		   
		   if(gap < p->v*1.1)
			if ( uniform(0.,1.,&seed) <= 0.9) 
		   		p->v=max(p->v-traffic.vunit, 0);
	    	   break;

	    default:
		    fprintf(stderr,"Unkown traffic model...\n");
		    exit(1);
	}


	p=p->next;
      } while (p!=(*last)->next);
  } // if(traffic.model > TRAFFIC_NBNA)


  oldest = p = (*last)->next;
  do {
    if(traffic.model > TRAFFIC_NBNA){    
       p->x+=p->v*traffic.step;    
       p->position=p->x*traffic.cell;
    }else{
       p->x=p->v*(current_time-p->arrival);    
       p->position=p->x;
    }
    //DBG_PL(UpdatePosition,"id:%d p:pos %f p:x:%f p:v:%f"),p->id,p->position,p->x,p->v DBG_PR;
    p=p->next;
  } while (p!=(*last)->next);
 
    
  
//  DBG_X(UpdatePosition);
}




Car *new_car(void)
{
#define BUNCHSIZE 60
 Car *p;
 int i;
 if (free_cars==NULL)
  {
    for (i=0; i<BUNCHSIZE ; i++)
     {
       if ((p=(Car *)malloc(sizeof(Car)))==NULL)
         { 	
	   fprintf(stderr,"Memory allocation failed in new_car()\n");
           exit(1);
         }
       p->arrival = 0;
       p->v=0;
       p->x=0;
       p->position=0.0; 
       in_list_car(&free_cars,p);
     }
  }
 return(out_list(&free_cars));  /* Ritorno il primo elemento della 
                                      Free List */
#undef BUNCHSIZE
}



void release_car(Car *elem)
{
 
   elem->arrival = 0;	   
   elem->v=0;	   
   elem->x=0;	   
   elem->position=0.0;     
   elem->next = NULL;
   elem->prev = NULL;
    
   in_list_car(&free_cars,elem);
}


int carsComputing(Car **last,double XPosition) {
  Car *p;
  int carback=0;
      
  if ((*last)==NULL)
       return 0;
  p = (*last);
  
  do {
    if ((p->position >(STREET_LENGTH-SecurityArea)) && p->position<XPosition){
        carback++;
    }
    p=p->prev;
    } while (p!=(*last));

  return carback;
}
