/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#include "incl.h"

extern double negexp (double mean, long *seed);


static ght_hash_table_t *channel_hash;
static char hash_key[9];
static struct channel_struct {
	double time;
	int state;
       } *channel_value, channel_zero;

#define need_hash() \
	((channel.model == CHANNEL_GILBERT) || \
	 (channel.model == CHANNEL_GENERAL))


#define DEBUG_CHANNEL 0
#if     DEBUG_CHANNEL
#define dbg2(fmt,v1,v2) fprintf(stderr,fmt,v1,v2)
#define dbg1(fmt,var)   fprintf(stderr,fmt,var)
#define dbg0(fmt)       fprintf(stderr,fmt)
#else
#define dbg2(fmt,v1,v2) {}
#define dbg1(fmt,var)   {}
#define dbg0(fmt)       {}
#endif

static int channel_good = 0;
static int channel_bad = 0;

int channel_done() {
  if (need_hash())
	  ght_finalize(channel_hash);

  fprintf(fileout,"#Channel: P{Loss} BAD GOOD\n%f %d %d\n",
	(double)channel_bad/((double) channel_bad + channel_good),
	channel_bad,
	channel_good);
  return 0;
}




// should select proportionally to steady state behavior
#define channel_steady_state() \
	(CHANNEL_GOOD)

extern double uniform(double,double,long *);
extern double uniform01(long *);
extern double negexp(double,long *);
extern int geometric1(double,long *);

int __channel_error(int i, int j) {
int channel_state = 0;

     //===================================================================================
     //  bernoully channel                                                                                 
     //-----------------------------------------------------------------------------------
     	     double threshold = channel.perror, distance;

	     if( channel.model == CHANNEL_TRANSITIONAL ) {
	        if(j<0) {
		   // this is the accident, which should always
		   // trigger the first transmission
			channel_state = CHANNEL_GOOD;
			goto return_value;
		}
	
		distance = minDistance(find_car(0,i), find_car(0,j));
		distance *= traffic.cell;
		   

	     	if (distance < (1-channel.transition)*range) {
			channel_state = CHANNEL_GOOD;
			goto return_value;
		}	
		if (distance > (1+channel.transition)*range) {
			channel_state = CHANNEL_BAD;
			goto return_value;
		}	
		double a = 1/(range*channel.transition);
		double b = -a * ((1-channel.transition)*range);
		threshold = a*distance + b;		
	     }	
		 
	     channel_state = (uniform01(&seed)<threshold) ? CHANNEL_BAD : CHANNEL_GOOD;	
	     goto return_value;	

    
    return_value:
	if (channel_state) 
		channel_bad++;
	else
		channel_good++;

        histo_add(H_channel, channel_state);
    	return channel_state;
}


