/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "incl.h"

extern double uniform(double,double,long *);

/*

  d MUST be normalized over the tx range (d/R)
  v MUST be normalized over the maxmium (v/traffic.vmax) 

*/


#ifndef abs(n)
#define abs(n) \
	((n)<0 ? -(n) / (n))
#endif

/* either d or v */
double d2flood_time(double d, double v) {
   if ((algo==ALGO_D2FLOOD) || (d2flood_rule < 0)) 
	   return (double) 2.0*txTime+uniform(0,31,&seed)*SlotTime;
   
   switch (d2flood_rule) {   
	case 0:
	   return (double) 2.0*txTime+uniform(0,31,&seed)*SlotTime;
	   break;

   	case 1: 
	   return (double) txTime+ uniform(0,(1.0-d)*d2flood_win,&seed)*SlotTime;
	   break;

   	case 2: 
	   return (double) txTime+ uniform(0,(1.0-v)*d2flood_win,&seed)*SlotTime;
	   break;

   	case 3: 
	   return (double) txTime+ uniform(0,(1.0-d)*(1.0-v)*d2flood_win,&seed)*SlotTime;
	   break;

   	case 4: 
	   return (double) txTime+ uniform(0,min((1.0-d),(1.0-v))*d2flood_win,&seed)*SlotTime;
	   break;

   	case 5: 
	   return (double) txTime+ uniform(0,(2.0-d-v)*d2flood_win,&seed)*SlotTime;
	   break;

   	default: 
		printf("invalid d2flood_time rule!");
		exit(-1);			
   }	
}

double d2flood_pfwd(double d, double v) {
   return (double) 1.0-pow(1.0-d, d2flood_k+0.0);
}

/* combination of d and v */
double d3flood_time(double d, double v) {
       printf("invalid d3flood_time rule!");
       exit(-1);			
}

double d3flood_pfwd(double d, double v) {
double kv, hv;

   switch (d3flood_rule) {   
   	case -1:   
	   return (double) 1.0-pow(1.0-d, d2flood_k+0.0);
	   break;

	case 0:
	   return (double) Pflood_constant;
	   break;

	case 1:
	   kv =   (double) d2flood_k;
	   return (double) 1.0 - pow(1.0-d, kv);
	   break;

	case 2:
	   kv =   (double) d2flood_k;
	   return (double) 1.0 - pow(1.0-v, kv);
	   break;

   	case 3: 
	   return (double) 1.0 - pow(1.0-d, d2flood_k)*pow(1.0-v, d3flood_h);
	   break;

	case 4:
	   kv =   (double) 1.0 + (d2flood_k-1.0) * pow(v, d3flood_h);
	   return (double) 1.0 - pow(1.0-d, kv);
	   break;

	case 5:
	   hv =   (double) 1.0 + (d3flood_h-1.0) * pow(d, d2flood_k);
	   return (double) 1.0 - pow(1.0-v, hv);
	   break;

   	default: 
		printf("invalid d2flood_pfwd rule!");
		exit(-1);			
   
   }  
}



/*
//----------------------------------
// HISTORICALLY (AWSN'06), in wow.c:
//----------------------------------

if(algo == ALGO_D2FLOOD || algo == ALGO_D3FLOOD){
   DBG_PL(TxInit,"dist now:%f min:%f"),distanceNow, p->minDist DBG_PR;
   DBG_PL(TxInit,"(ptx:%f  p_rx:%f)"),p_tx->position,p->position DBG_PR;
   if(p_tx->position<p->position){
      p->minDist=0;			
   }else if((distanceNow<p->minDist && p_tx->position>p->position) || p->minDist==0){
      p->minDist=distanceNow;
   }
   // back the old times of AWSN, it was that way 
   Pflood = p->minDist/range;
   Pflood = (k_d2flood<0) ? 1.0 : 1.0-pow((1.0-Pflood),k_d2flood+0.0);
		   
#define P_D3FLOOD(v) \
	( (v)>0.99 ? 1.0 : min(1.0, 1-pow(((v)-0.99)/((v)*(v)-0.99),k_d2flood+0.0)) )

  if (algo == ALGO_D3FLOOD) 
    //rule1: 
    //Pflood = max(Pflood, P_D3FLOOD( p->v/traffic.vmax ));
    //rule2: 
    //Pflood = max(P_D3FLOOD( p->minDist/range ), P_D3FLOOD( p->v/traffic.vmax  ));
    //rule3: 
    //Pflood = 1.0-pow((1.0-p->minDist/range),1+(k_d2flood-1.0)*P_D3FLOOD( p->v/traffic.vmax ));
    //rule4: 
    //Pflood = 1.0-pow((1.0-p->minDist/range),(k_d2flood+0.0)*P_D3FLOOD( p->v/traffic.vmax ));
    //rule5: 
    //Pflood = 1.0-pow((1.0-p->minDist/range),1+(k_d2flood-1.0)*( pow(P_D3FLOOD( p->v/traffic.vmax ),2 )));
    //rule6: 
    //Pflood = 1.0-pow((1.0-p->minDist/range),(k_d2flood+0.0)*(p->v/traffic.vmax)*(p->v/traffic.vmax));
    //rule7: 
    //initially with pow(v,2)
      Pflood = 1.0-pow((1.0-p->minDist/range),1+(k_d2flood-1.0)*pow(p->v/traffic.vmax,4));

 DBG_PL(TxInit,"min dist:%f  pf:%f"), p->minDist,Pflood DBG_PR;
}

*/

