
/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/

/* Based on, see below for original copyrigth */

/*  
 * Copyright (c) 2001-2002 by the Protocol Engineering Lab, U of Delaware
 * All rights reserved.
 *
 * Armando L. Caro Jr. <acaro@@cis,udel,edu>
 *
 * Armando's debugging output system. 
 * Based on a previous version written by Phill Conrad and Ed Golden.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the University nor of the Laboratory may be used
 *    to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * @(#) $Header: /m/pel/cvs/ns-allinone-2.1b8.sctp/ns-2.1b8/sctpDebug.h,v 1.18 2002/11/15 01:52:08 acaro Exp $ (UD/PEL)
 */

#ifndef Debug_h
#define Debug_h

//#if(DEBUG)

/* debug variable format:  __dbg_FunctionName
 */
#define __dbg_Warning                            0x00000002
#define __dbg_Accident                           0x20000000
#define __dbg_RxInit                             0x00000004
#define __dbg_RxEnd                              0x00000008

#define __dbg_UpdatePosition                     0x00000010
#define __dbg_FindCar                            0x00000010
#define __dbg_TxInit                             0x00000010

#define __dbg_TxEnd                              0x00000020
#define __dbg_TryAgain                           0x00000020
#define __dbg_simula                             0x00000020
#define __dbg_CarArrival                         0x00000020
#define __dbg_CarExit                            0x00000020

#define __dbg_outlist                            0x00000040
#define __dbg_inlist                             0x00000040
#define __dbg_findcoll                           0x00000040
#define __dbg_insert_event                       0x00000040
#define __dbg_Wait                       	0x00000040
#define __dbg_HelloRx                       	 0x00000040
#define __dbg_HelloRxEnd                       	 0x00000040
#define __dbg_HelloTx                       	 0x00000040
#define __dbg_HelloTxEnd                       	 0x00000040
#define __dbg_Hello                       	 0x00000040
#define __dbg_Measure                      	 0x00000040
#define __dbg_Update                      	 0x00000040

/**************************** DEBUGGING MACROS **************************/

/* For Function Entry 
 *
 * example usage:   DBG_I(SomeFunctionName);
 */
#define DBG_I(funcName) \
        if(fhpDebugFile != NULL) /* make sure output file is open! */ \
          if (uiDebugMask & __dbg_##funcName) \
	    {fprintf(fhpDebugFile, "[%s] <I> \n", #funcName);}

/* For Printing Details
 *
 * example usage:
 *          DBG_PL(SomeFunctionName, "testing") DBG_PR;
 *          DBG_PL(SomeFunctionName, "current someValue=%d"), someValue DBG_PR;
 */
#define DBG_PL(funcName, format) \
        if(fhpDebugFile != NULL) /* make sure output file is open! */ \
          if (uiDebugMask & __dbg_##funcName) \
	    fprintf(fhpDebugFile, "[%s] " format "\n", #funcName
#define DBG_PR )

/* For Calling Debugging Functions
 * 
 * example usage:
 *          DBG_F(SomeFunctionName, SomeDebugFunction(param1, parm2, ...) );
 *
 * Note: both SomeFunctionName and SomeDebugFunction debug bits must be set
 */
#define DBG_F(funcName, debugFuncCall) \
        if(fhpDebugFile != NULL) /* make sure output file is open! */ \
          if (uiDebugMask & __dbg_##funcName) \
	    debugFuncCall

/* For executing blocks of code only if debug bit is set
 *
 * example usage:
 *          DBG_IF(SomeFunctionName)
 *            {
 *              // block of code goes here
 *            }
 */
#define DBG_IF(funcName) \
        if(fhpDebugFile != NULL) /* make sure output file is open! */ \
          if (uiDebugMask & __dbg_##funcName)

/* For Function Exit
 *
 * example usage:    DBG_X(SomeFunctionName);
 */
#define DBG_X(funcName) \
        if(fhpDebugFile != NULL) /* make sure output file is open! */ \
          if (uiDebugMask & __dbg_##funcName) \
	    {fprintf(fhpDebugFile, "[%s] <X> \n", #funcName);}

#define DBG_FOPEN() \
        if(fhpDebugFile == NULL) /* only open file once! */ \
          { \
	     if(iDebugFileIndex >= 0) \
	       { \
		 char cpFileName[256]; \
		 sprintf(cpFileName, "debug.SctpAgent.%d", iDebugFileIndex); \
		 fhpDebugFile = fopen(cpFileName, "w"); \
	       } \
	     else \
	       fhpDebugFile = stdout; \
	  }

//#else  /* DEBUG is not defined, then preprocess these macros out! */

//#define DBG_I(bitmask)            { /* empty */ }

/* This is a HACK!!! ...but who cares, it's only debugging and gcc should strip
 * it out anyway.
 */
//#define DBG_PL(funcName, format)  if(0) printf(""
//#define DBG_PR                    )

//#define DBG_F(funcName, debugFuncCall)  { /* empty */ }

//#define DBG_IF(funcName)          if(0)

//#define DBG_X(funcName)           { /* empty */ }

//#define DBG_FOPEN()               { /* empty */ }

//#endif
#endif
