/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/
#ifndef DEFS_H
#define DEFS_H


/* 
 *  From the id, find the corresponding car structure
 *
 *	 Car *find_car(Car **last,int id) {
 *	    assert(id < MAX_CAR);
 *	    return fast_car_search[id];
 *	} 
 * 
 */
#define find_car(dummy,id) \
	(fast_car_search[id])


#define knospeDistance(a, b) \
	(( (a)->x - (b)->x - (CAR_LEN-1)) + \
	(( (a)->x > (b)->x + (CAR_LEN-1)) ?  0 : STREET_LENGTH/traffic.cell))

#define naschDistance(a, b) \
	(( (a)->x-(b)->x-1) + (( (a)->x > (b)->x) ?  0 : STREET_LENGTH/traffic.cell))
  	

#define carDistance(a, b) \
		naschDistance(a, b)

#define minDistance(a, b) \
	(carDistance(a,b) < carDistance(b,a) ? carDistance(a,b) : carDistance(b,a))

//Algorithms 
#define ALGO_NONE	0
#define ALGO_COVERAGE	1
#define ALGO_PFLOOD	2
#define ALGO_D2FLOOD	3
#define ALGO_HELP	4
#define ALGO_HYBRID	5
#define ALGO_D3FLOOD	6



//Traffic Simulator -- microscopic car behavior
#define TRAFFIC_NBNA  		0
#define TRAFFIC_NASCH 		1
#define TRAFFIC_NASCH_MINISTEP	2
#define TRAFFIC_GIPPS		3
#define TRAFFIC_GIPPS_MINISTEP	4
#define TRAFFIC_VDR		5
#define TRAFFIC_VDR_MINISTEP	6
#define TRAFFIC_TSQ		7
#define TRAFFIC_TSQ_MINISTEP	8
#define TRAFFIC_FUKUI		9
#define TRAFFIC_FUKUI_MINISTEP	10
#define TRAFFIC_TOCA		11


#define MINI_STEP
#define MAX_CAR    1000
#define MIN_CYCLES 500


//===================================================================================
// giugio: Wireless channel model
//-----------------------------------------------------------------------------------
#define CHANNEL_ERROR_FREE      0
#define CHANNEL_BERNOULLI       1
#define CHANNEL_TRANSITIONAL    2
#define CHANNEL_GILBERT         3
#define CHANNEL_GENERAL         4

#define CHANNEL_GOOD  0
#define CHANNEL_BAD   1
#define CHANNEL_LBAD  2
#define CHANNEL_SBAD  3

#define CHANNEL_HASH_SIZE 10000

//===================================================================================
// Hybrid Algo (./wow -y val => par_y == val)
//-----------------------------------------------------------------------------------
#define HYBRID_D2FUNC	0
#define HYBRID_ALWAYS	1
#define HYBRID_INVRAND	2
#define HYBRID_XOR_D2FUNC	3
#define HYBRID_XOR_ALWAYS	4
#define HYBRID_XOR_INVRAND	5

//===================================================================================
// mp2p estimate                                                                                  
//-----------------------------------------------------------------------------------
//#define HELLO_ESTIMATE
#define ERROR_ESTIMATE

#ifdef ERROR_ESTIMATE
//I need HELLO_ESTIMATE to be turned on
#ifndef HELLO_ESTIMATE  	       
#define HELLO_ESTIMATE
#endif // HELLO_ESTIMATE
#endif //ERROR_ESTIMATE

  
#define min(a,b) (((a)<(b))?(a):(b)) 
#define max(a,b) (((a)>(b))?(a):(b)) 

// since we use 1 unit of time = 1ns,
// our radio signal travels at the speed
// of 3 meters every nanosecond
#define LIGHT_SPEED 3/10

#define ONE_SECOND 1.0e6
#define MAX_RHO    250	
#define SLIDING_W  60.0*ONE_SECOND 

#define SlotTime 20
#define STAT_ALMOST_ON 2
#define STAT_ON  1
#define STAT_OFF 0


typedef struct car_unit Car;
typedef enum St {DRIVE=1, TX, RX, COLL, WAITING} State;  


struct car_unit
  { 
    Car *next;
    Car *prev;    
    int id;        /* car id*/
    double arrival; /* arrival time*/
    double exitTime; /* exitTime for Poisson traffic*/
    double position;
    double d;
    double v;
    double lastv;
    double maxErrV;
    int vchange;
    double x;   
    double Pf;
    int warning;   /*warning already received*/
    int collided;  /*the car received a collided info*/
    int pkt_inrx;  /*number of packets in reception 
                   (if more than one ->collision)*/
    int NEW;
    State CAR_State;   
    double TxEND;
    double RxEND;
    int relay;
    int firstWait;
    double minDist;


//this choice doens't scale at all    
    double myPos[MAX_CAR];   // when I received i-th HELLO
    double mySpeed[MAX_CAR];
    double vicPos[MAX_CAR];
    double vicSpeed[MAX_CAR];
    double HelloTxTime[MAX_CAR];
    int    HelloC[MAX_CAR];
    int    MyHelloC;
    double minPos;
    
    int rx;
    double ChickFactor; // dist/range
    
    int phase; // to implement an exponential backoff for the wait-time
    double last_rx_packet_end; // time at which the reception of the last (0correcly received) packet ended
  };

 typedef struct _EventParam {
    Car    *ptr;
    int    id;
    int idTx; 
    double vTx,xTx,tTx;
    double Pf;
 } EventParam;


#endif
