/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include "incl.h"

// extern FILE *fhpDebugFile;
// extern int uiDebugMask;     /*if =-1 debug=ON*/

Event *free_events = NULL;  /* Puntatore alla mia Free List di eventi */

/* 
**  Funzione:  void insert_event(Event **last, Event *elem)
**  Parametri: Event **last  - Puntatore di riferimento alla lista (l'ultimo
**                             elemento)
**             Event  *elem  - Record di tipo Event da inserire
**  Ritorno:   nessuno
**  Descrizione: Inserisce 'elem' nella Event List referenziata da **last
*/
void insert_event(Event **last, Event *elem)
{
  Event *p;
  if (elem==NULL) return;       /* Non c'e' nulla da inserire */

  //DBG_PL(insert_event,"insert elem:%f (type:%d - car:%d))"),elem->time, elem->type,elem->car DBG_PR;
  /* L'Event List e' vuota: elem diventa l'Event List */
  if ((*last)==NULL)
    {
       (*last) = elem;
       (*last)->next = elem;
       (*last)->prev = elem;
       return;
    }

  /* 
  ** elem e' schedulato dopo l'ultimo evento presente in lista:
  ** viene inserito in fondo e diventa il nuovo last
  */
  if (elem->time >= (*last)->time)
    {
       elem->prev = (*last);
       elem->next = (*last)->next;
       ((*last)->next)->prev = elem;
       (*last)->next = elem;
       (*last) = elem;
    }
  else
  /* 
  ** cerco la posizione di elem, procedendo dal fondo della lista 
  ** ovvero seguendo il puntatore prev
  */
    {
       p = (*last);
       while (elem->time < p->time && p->prev!=(*last))
         p = p->prev;
	 
       //DBG_PL(insert_event,"insert:%f (type:%d - car:%d))"),elem->time,elem->type,elem->car DBG_PR;
       //DBG_PL(insert_event,"pointer:%f (type:%d - car:%d))"),p->time,p->type,p->car DBG_PR;
       if (elem->time < p->time)
       { 
           //DBG_PL(insert_event,"min [delta %le]"), elem->time - p->time DBG_PR;
		   /* p e' il primo elemento schedulato dopo elem: 
		      inserisco elem subito prima */
          if((elem->par.id==p->par.id)  && 
	     (((elem->type==3)&&(p->type==5)) || 
	      ((elem->type==8)&&((p->next)->type==10)))) {
	      elem->prev = p;
              elem->next = p->next;
              (p->next)->prev = elem;
              p->next = elem;

	  }else{	  
              elem->prev = p->prev;
              elem->next = p;
              (p->prev)->next = elem;
              p->prev = elem;
          }
       }  else {
		   /* elem deve essere il primo evento della lista:
		      inserisco elem dopo p (che e' *last) */
          //DBG_PL(insert_event," maggiore") DBG_PR;
          //DBG_PL(insert_event,"prec:%f (type:%d - car:%d)"),(p->next)->time,(p->next)->type,(p->next)->par.id DBG_PR;
          if((elem->par.id==(p->next)->par.id) && ((elem->type==3&&(p->next)->type==5) || 
	  (elem->type==8&&(p->next)->type==10))){
	      elem->prev = p->next;
              elem->next = (p->next)->next;
              ((p->next)->next)->prev = elem;
              (p->next)->next = elem;

	  }else{	  
              elem->prev = p;
              elem->next = p->next;
              (p->next)->prev = elem;
              p->next = elem;
          }
      }           
   }
         
  return;
}

/* 
**  Funzione:  Event *get_event(Event **last)
**  Parametri: Event **last  - Puntatore di riferimento alla lista (l'ultimo
**                             elemento)
**  Ritorno:   Puntatore all'evento estratto
**  Descrizione: Ritorna il prossimo evento da schedulare, estraendolo dalla
**               cima della lista. Torna NULL se l'Event List era vuota
*/
Event *get_event(Event **last)
{
  Event *p;
  if ((*last)==NULL) return NULL;   /* L'Event List e' vuota */
  if ((*last)->next==(*last))
    {
	  /* Sto estraendo l'unico elemento della Event List */
      p = (*last);
      
      // printf("evento(unico):  time=%f type:%d\n",p->time,p->type );
      
      (*last) = NULL;
    }
  else
    {
	  /* Estraggo il primo elemento (successore di last) */
      p = (*last)->next;
      
      // printf("evento:  time=%f type:%d\n",p->time,p->type );
      
      (p->next)->prev = (*last);
      (*last)->next = p->next;
    }
  return p;
}

/* 
**  Funzione:  Event *new_event(void)
**  Parametri: Nessuno
**  Ritorno:   Puntatore ad un nuovo evento
**  Descrizione: Ritorna un nuovo evento.
**  Effetto collaterale: Inserisce o rimuove elementi dalla Free List globale
*/
Event *new_event(void)
{
#define BUNCHSIZE 60
 Event *p;
 int i;
 if (free_events==NULL)
  {
    /* La Free List e' vuota, alloco un certo numero di record e
	   li inserisco nella lista
	*/
	/* Codice inutile?
    if ((p=(Event *)malloc(sizeof(Event)))==NULL)
         { printf("Error  : Memory allocation error\n");
           exit(1);
         }
    p->time = (Time)0;
    insert_event(&free_events,p);
	*/
    for (i=0; i<BUNCHSIZE ; i++)
     {
       if ((p=(Event *)malloc(sizeof(Event)))==NULL)
         { 	
	 	fprintf(stderr,"Memory allocation failed in new_event()\n");
		exit(1);
         }
       p->time = (double)0;
       // printf("genero elem:%f (type:%d - car:%d))\n",p->time, p->type,p->car);
       insert_event(&free_events,p);
     }
  }
 return(get_event(&free_events));  /* Ritorno il primo elemento della 
                                      Free List */
#undef BUNCHSIZE
}

/* 
**  Funzione:  void release_event(Event *elem)
**  Parametri: Evento da rilasciare
**  Ritorno:   Nessuno
**  Descrizione: Rilascia la struttura dati dell'evento, inserendola nella 
**               Free List
*/
void release_event(Event *elem)
{
// extern Event *free_events;
 elem->time = (double)MAX_TIME;    /* Cosi' il record va subito in fondo alla 
                                    Free List */
 insert_event(&free_events,elem);
}
