/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "incl.h"
 
struct _help {
       double xA,  xB,  xC;   // real postition
       double xAh, xBh, xCh;  // HELLO-postition
       double xAest,  xBest,  xCest;   // estimated postition
       double exA, exB, exC;  // estimation error 

       // vB it's the speed of the car that must decide
       double vA,  vB,  vC;   // actual speed
       double vAh, vBh, vCh;  // HELLO-speed
       double evA, evB, evC;  // estimation error 

       double vAB, vBC; 	//vA-vB, vB-vC
       double DeltaTA,DeltaTB,DeltaTC;
       
       int failing; 
       int neighbor_cars;
       double coverage;
} help;



#define BoundaryTest(x)  \
	    ( x>(STREET_LENGTH/traffic.cell) ? x-(STREET_LENGTH/traffic.cell) : x )

#define ContinuousApplyEstimate(x,v,t)  \
	    BoundaryTest(( (x*traffic.cell) + (v*traffic.cell/traffic.step)*t )/traffic.cell)

#define ApplyEstimate(x,v,t)  \
	    BoundaryTest( ( (x) + (v) * floor( (t) /traffic.step)) )
	    
#define EstimateNeighborPosition(i) \
	traffic.cell * \
	ApplyEstimate(p_rx->vicPos[i], p_rx->vicSpeed[i], \
		 (timeAccident-1.0-p_rx->HelloTxTime[i])/ONE_SECOND)
		    
#define EstimateMyPosition(i) \
	traffic.cell * \
	ApplyEstimate(p_rx->myPos[i], p_rx->mySpeed[i], \
		 (timeAccident-1.0-p_rx->HelloTxTime[i])/ONE_SECOND)

#define ValidNeighbor(i) \
	((p_rx->HelloTxTime[i]>0) && ((HelloCounter-p_rx->HelloC[i])<= MAX_CICLI_HELLO))

/*
#define isNeighbor(i)  \
	((p_rx->HelloTxTime[i]>0)&&((HelloCounter-p_rx->HelloC[i])<= MAX_CICLI_HELLO))
	*/
#define isNeighbor(i)    \
	(p_rx->HelloTxTime[i]>0)
	
#define cvtV(v)	\
	((v)*traffic.cell/traffic.step*3.6)

#define cvtX(x) \
	((x)*traffic.cell)

extern double uniform01(long *);


double help_error_margin(EventParam par, int id_farmost,int in_rx_range);
void help_arrow_art(Car *p_rx, int id_farmost);
void help_error_estimate(EventParam par, int id_farmost);



//===================================================================================
// help_rx_end                                                                                  
//-----------------------------------------------------------------------------------
static long Hybrid_beaconed = 0;
static long Hybrid_beaconless = 0;

void help_rx_end(EventParam par) {
double dist,posVicino;
int i,valid;	
int id_farmost  = -1;
int in_tx_range=0;
int car_preceeding=0, car_overtaking=0;	  
Car *p_rx;

	help.neighbor_cars = 0;

p_rx = par.ptr;
p_rx->minDist=STREET_LENGTH;
DBG_PL(RxEnd,"minDist:%f   my_pos:%f (%f)"),p_rx->minDist,p_rx->position,p_rx->x DBG_PR;		  

   if(par.idTx!=-1)
      help.coverage=par.xTx*traffic.cell-range;
   else
      help.coverage=par.xTx-range;

    for (i=0;i<MAX_CAR;i++) {
	valid=0;

	if(! ValidNeighbor(i)) {
	    if(p_rx->HelloTxTime[i]>0) {
	       DBG_PL(RxEnd,"scartata") DBG_PR;		      
	       DBG_PL(RxEnd,"No longer VALID %d's shitty neighnor(%d) %d p_rx->HelloC[i]"), 
    			      p_rx->id, i, p_rx->HelloC[i]
			      DBG_PR;
	    }
	    p_rx->vicPos[i]=0;
	    p_rx->vicSpeed[i]=0;
	    p_rx->HelloTxTime[i] =0; 

	    /*ATTENZIONE!!!!!!*/  
	    //p_rx->minPos=STREET_LENGTH;   

       }else{
           help.neighbor_cars++;
       
	   DBG_PL(RxEnd,"%d -- C:%d (max:%d) HtxTime:%f  speed:%f pos:%f"),i,p_rx->HelloC[i],HelloCounter,p_rx->HelloTxTime[i],p_rx->vicSpeed[i],p_rx->vicPos[i] DBG_PR;		      

	    // estimate neighbor position
	   posVicino = EstimateNeighborPosition(i);

	    // boundaries...
	   dist=p_rx->position-posVicino;
	   if(fabs(dist)>4000){
	      if(p_rx->position>posVicino)
		  dist=p_rx->position-posVicino-STREET_LENGTH;
	      else
		  dist=p_rx->position+STREET_LENGTH-posVicino;			    		
	   }
	   DBG_PL(RxEnd,"dist:%f vicinoPos:%f"),dist,posVicino DBG_PR;
	   DBG_PL(RxEnd,"vicPos_atHello:%f my_atHello:%f"),p_rx->vicPos[i],p_rx->myPos[i] DBG_PR;

	    // no overtakers, please
	   if ( p_rx->vicPos[i]<p_rx->myPos[i] ) {
	      if (dist<0) car_overtaking++;
	      car_preceeding++;
	   }

           if((dist>0) &&  (p_rx->position-dist)>help.coverage){
	          in_tx_range++;
	          DBG_PL(RxEnd,"%d in_range (tot:%d)"),i,in_tx_range DBG_PR;
	          
	   }
	   
	   if((dist<p_rx->minDist) && (dist>0)) {
	      p_rx->minDist=dist;
	      id_farmost = i;
 	   }   
       }
   }
   DBG_PL(RxEnd,"TxPos:%f  my:%f  mindist:%f (car:%d)"),par.xTx*traffic.cell,\
   p_rx->position,p_rx->minDist,id_farmost DBG_PR;
  
  
      
   // calculate systsem parameter
   double error_margin;
   help_error_estimate(par, id_farmost);
   error_margin = help_error_margin(par, id_farmost,in_tx_range);


  
   DBG_PL(RxEnd,"coverage:%f  stimata:%f ErrorMargin:%f"),help.coverage,(p_rx->position-p_rx->minDist),error_margin DBG_PR;
   p_rx->relay= (help.coverage + error_margin)>(p_rx->position-p_rx->minDist);		                    


  //===================================================================================
  //  Hybrid from 0.7x                                                                                   
  //-----------------------------------------------------------------------------------
 
   if ((algo == ALGO_HYBRID) && (uniform01(&seed) > (1.0-Hybrid) )) {
       // then, rebroadcast with D2FLOOD's probability function
       // see ASWN'06 for details
      
       Hybrid_beaconless++;      
       // printf("X%f\n", (double) Hybrid_beaconed  / ((double) Hybrid_beaconed + Hybrid_beaconless)); 

       int beaconed_relay   = p_rx->relay;
       int beaconless_relay = 0;

	
       // printf("X: %e   D(): %e\n", X, carDistance(par.idTx, par.id));
       
       switch ((int)par_y) {
          case HYBRID_XOR_D2FUNC:
	        beaconed_relay = 0;	  
          case HYBRID_D2FUNC: 
	        {
         	  // -1 to cope with accident, which is at Danger Point, thus x==0
       	          double X = fabs( ((par.idTx==-1) ? 0 : (find_car(0,par.idTx))->position) - p_rx->position);
       		  double P = d2flood_pfwd(X/range, help.vB/traffic.vmax);
		  beaconless_relay = (uniform01(&seed) < P);
		}
		break;
   
          case HYBRID_XOR_ALWAYS:
	        beaconed_relay = 0;	  
          case HYBRID_ALWAYS:
	   	beaconless_relay = 1;
		break;
		
          case HYBRID_XOR_INVRAND:
	        beaconed_relay = 0;	  
          case HYBRID_INVRAND:
	        // so that, if only 10% of the time the HYBRID func is used,
		// then w.p. 90% it will augment the relay set
	  	beaconless_relay = (uniform01(&seed) < (1.0-Hybrid));
	  	break;
		
	  default:
	  	printf("par_y is set to an HYBRID_unkown value");
		exit(1);	
	}
       
        p_rx->relay = beaconed_relay || beaconless_relay; 		       
       
       //===============================================================
       //  Algo             rho  #msg     pinfo     failing_in_decision
       //  HELP             20   13.800   0.3312    0.0223
       //  0.9HELP+0.1D2FD  20   17.079   0.3325    0.0196
       //  D2FD             20   26.799   0.3342    0.0055

   } else {
       Hybrid_beaconed++;
   }
   
  //===================================================================================

   if(H_alfa >=0){   
      histo_add(H_alfa, p_rx->relay);	
   }	
 

   DBG_PL(RxEnd,"relay:%d"),p_rx->relay DBG_PR;	
}              





//===================================================================================
//  error_margin                                                                                   
//-----------------------------------------------------------------------------------

double help_error_margin(EventParam par, int id_farmost, int in_tx_range) {
double ErrorMargin = epsilon;

   switch( AdaptiveMargin ) {
      case 0: ErrorMargin = epsilon; 

     /* the faster you go, the more you increase the error margin */
      case 1: ErrorMargin = epsilon * (1.0 + (help.vB/traffic.vmax)); 

/*
      case 1: DBG_PL(RxEnd,"my_at_h:%f  stimata:%f  real:%f  errore:%f"),help.xBh,help.xBest,help.xB,help.exB DBG_PR;
      ErrorMargin = (help.exB < 0) ? epsilon : 4.0; break;

      case 2: DBG_PL(RxEnd,"my_at_h:%f  stimata:%f (in range:%d)  real:%f  errore:%f"),help.xBh,help.xBest,in_tx_range, help.xB,help.exB DBG_PR;
      ErrorMargin = (help.exB < 0 && in_tx_range<2) ? epsilon : 2.0; break;

      case 3: DBG_PL(RxEnd,"my_at_h:%f  stimata:%f (in range:%d)  real:%f  errore:%f"),help.xBh,help.xBest,in_tx_range, help.xB,help.exB DBG_PR;
      ErrorMargin = (help.exB < 0 && in_tx_range<2) ? (-help.exB*traffic.cell) : 2.0; break;

      // when I go slower than expected, I'm jammed
      case 101: ErrorMargin = (help.evB < 0) ? 2*epsilon : 0.0; break;

      // when I go slower than expected, I'm jammed
      // especially is A is faster than me, cos
      // otherwise I'd follow him closer and at roughly
      // the same speed
      case 102: ErrorMargin = ((help.evB < 0) && (help.vAB > 0)) ? 2*epsilon : 0.0; break;

      // sloppier than before: more msgs, higher pinfo
      case 103: ErrorMargin = ((help.evB < 0) || (help.vAB > 0)) ? 2*epsilon : 0.0; break;

      // sloppier than before
      case 104: ErrorMargin = ((help.evB < 0) && (help.vAB > 0)) ? 2*epsilon :
	                    ((help.evB < 0) || (help.vAB > 0)) ? epsilon :
			     epsilon/2; 
			     break;
*/
   }   


   histo_add(H_EM, ErrorMargin);

#ifndef MINI_STEP
  ErrorMargin *= cell;
#endif
  return ErrorMargin;
}

/*===================================================================================
// get_ABC info
//-----------------------------------------------------------------------------------
 
						  fartest B's neighbor 
	       tx_sender        tx_receiver       within A tx_range  
#  <-veh          A                 B                 C
#-----------------+-----------------+-----------------+----------
#   msg->			B is receiving 
				A's warning and
				decides what to do	


*/

//===================================================================================
// evaluate the the extent of the error estimate 
//-----------------------------------------------------------------------------------
void help_error_estimate(EventParam par, int id_farmost) {
Car *p_rx = par.ptr;

 if (id_farmost>=0) {
    Car *neighbor = find_car(&queue,  id_farmost);  

    // B estimates that A's coverage is enough for C
    int B_is_relay = help.coverage > (p_rx->position-p_rx->minDist);		                    

    // is he wrong ?
    int C_is_covered =  help.coverage < neighbor->position;

    help.failing = (!B_is_relay) && (!C_is_covered);

 
    //===================================================================================
    //   vehichle A      : hello_tx sender                                                                         
    //-----------------------------------------------------------------------------------
    int hello_tx_id = par.idTx;
     Car *myself = p_rx;
    Car *hello_tx=find_car(&queue, hello_tx_id);

   if ( myself==NULL||hello_tx==NULL)
	return;
	
     // at accident time, one withing the accident txrange 
     // decides to broadcast, and I'm in between, so I do
     // not rebroadcast
//      if ( hello_tx->x < myself->x ) 
// 	return;

   // number of position update since HELLO tx
      if(myself->HelloTxTime[hello_tx_id]>0){
	  help.vAh = myself->vicSpeed[hello_tx_id];
	  help.xAh = myself->vicPos[hello_tx_id];	      

	  help.DeltaTA = (timeAccident-myself->HelloTxTime[hello_tx_id])/ONE_SECOND;
	  help.xAest = ApplyEstimate(help.xAh,help.vAh,help.DeltaTA);       

	  help.xA  = hello_tx->x;	      
	  help.vA  = hello_tx->v;	      
	  help.exA = help.xA - help.xAest;

	  if(fabs(cvtX(help.exA))>500){
	     if(help.xA<help.xAest)
	       help.exA=help.xA + STREET_LENGTH/traffic.cell/1.0 - help.xAest;
	     else
	       help.exA=help.xA - STREET_LENGTH/traffic.cell/1.0 - help.xAest;
	  }
	  help.evA = help.vA - help.vAh;
      }
     //===================================================================================
     //   vehichle B      : hello_tx receiver
     //-----------------------------------------------------------------------------------
     help.xBh = myself->myPos[id_farmost]; // at reception time of hello[hello_tx_id];  
     help.vBh = myself->mySpeed[id_farmost];

     help.DeltaTB = (timeAccident - myself->HelloTxTime[id_farmost])/ONE_SECOND;
     help.xBest = ApplyEstimate(help.xBh,help.vBh,help.DeltaTB);
     help.xB  = myself->x;		
     help.vB  = myself->v;     		
     help.exB = help.xB - help.xBest;
     if(fabs(cvtX(help.exB))>500){
	 if(help.xB<help.xBest)
	   help.exB=help.xB + STREET_LENGTH/traffic.cell/1.0 - help.xBest;
	 else
	   help.exB=help.xB - STREET_LENGTH/traffic.cell/1.0 - help.xBest;
     }
     help.evB = help.vB - help.vBh;


     //===================================================================================
     //   vehichle C      : farmost hello_tx receiver as estimated by B
     // ANTICIPATED:        Car *neighbor = find_car(&queue,  id_farmost);  
     //-----------------------------------------------------------------------------------   	  

       help.vCh = myself->vicSpeed[id_farmost];
       help.xCh = myself->vicPos[id_farmost];
       help.DeltaTC = (timeAccident - myself->HelloTxTime[id_farmost])/ONE_SECOND;
       help.xCest = ApplyEstimate(help.xCh,help.vCh,help.DeltaTC);
 
       help.xC  = neighbor->x;
       help.vC  = neighbor->v;
       help.exC = help.xC - help.xCest;
       if(fabs(cvtX(help.exC))>500){
	   if(help.xC<help.xCest)
	     help.exC=help.xC + STREET_LENGTH/traffic.cell/1.0 - help.xCest;
	   else
	     help.exC=help.xC - STREET_LENGTH/traffic.cell/1.0 - help.xCest;
       }
       help.evC = help.vC - help.vCh;
       
       double xAB=help.xA-help.xB;
       if(fabs(cvtX(xAB))>500){
	   if(help.xA<help.xB)
	     xAB=help.xA + STREET_LENGTH/traffic.cell/1.0 - help.xB;
	   else
	     xAB=help.xA - STREET_LENGTH/traffic.cell/1.0 - help.xB;
       }
       double xBC=help.xB-help.xC;
       if(fabs(cvtX(xBC))>500){
	   if(help.xB<help.xC)
	     xBC=help.xB + STREET_LENGTH/traffic.cell/1.0 - help.xC;
	   else
	     xBC=help.xB - STREET_LENGTH/traffic.cell/1.0 - help.xC;
       }


       double posVicino = EstimateNeighborPosition(neighbor->id);

       double error=neighbor->position - posVicino;
       DBG_PL(RxEnd,"error:%f  (real:%f  stimata:%f)"),error,neighbor->position,posVicino DBG_PR;	
       if(fabs(error)>4000){
	 //printf("error:%f  (real:%f  stimata:%f)\n",error,neighbor->position,posVicino);	
	 //quit(1);
	 if(neighbor->position<posVicino)
	   error=neighbor->position + STREET_LENGTH - posVicino;
	 else
	   error=neighbor->position - STREET_LENGTH - posVicino;
	 DBG_PL(RxEnd,"error:%f  (real:%f  stimata:%f)"),error,neighbor->position,posVicino DBG_PR;	
// 		   if(fabs(error)>4000)
// 		     quit(1);

       }

       histo_add(H_AerrVehicle, fabs(error));

       if (error<0) 
       histo_add(H_AoverErrVehicle, fabs(error));

       if (!C_is_covered) {
	   histo_add(H_errVehicle, fabs(error));

	   if (error<0) 
	   histo_add(H_overErrVehicle, fabs(error));

	    if (help.failing) {
	      histo_add(H_errFailing, fabs(error));
	      if (error<0) 
		 histo_add(H_overErrFailing,  fabs(error));
	    }
       }     

     }			
}


