/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "incl.h"

#ifndef HISTO_H
#include "histo.h"
#endif

//#ifndef round(num)

char buf[128];
struct histo* histo_array[100];
static int histoNum=0;
char prefix[1024];
char postfix[1024];

void histo_setup(char *pre, char  *pos) {
  sprintf(prefix,"%s",pre);
  sprintf(postfix,"%s",pos);
}

void __histo_add(struct histo *h, double val, int dbg);
int __histo_dump(struct histo *h, int print_cdf);

/* user callble functions */
struct histo* histo_get(int ID) { return histo_array[ID]; }
int histo_dump(int ID, int print_cdf) { return __histo_dump(histo_array[ID], print_cdf); }

void histo_add(int ID, double val) {  
	__histo_add( histo_array[ID], val, ID==H_errVehicle ); 
}

double histo_avg(int ID) { return histo_array[ID]->avg; }

int histo_newN(char *name, double min, double max, int num) {
  return histo_newD(name, min, max,  (max - min) / (double) num);
}

int histo_newD(char *name, double min, double max, double delta) {
  struct histo *h;
  int num, *tdata;

  /* printf("Intervallo [%f:%f] con %d sottointervalli di %f ciascuno\n", min, max, num_col, delta); */
  num = (int) ceil((max - min) / delta);
  if ( delta <= 0 || num <= 0 )  {
      // (max != (min + (num * delta))) || delta <= 0 || num <= 0)  {
      fprintf(stderr,"Error: wrong param when creating ``%s'' histogram \n", name);
      fprintf(stderr,"\n\tmin: %f\n\tmax: %f\n\tnum: %d\n\tdel: %f\n\tchk %f\n",
                     min,max,num,delta,(min + (num * delta)));
  }
  num +=2;
 
  tdata = (int *) malloc(num * sizeof (int));
  int i; for(i=0; i<num; i++) tdata[i]=0;  
  h= (struct histo *) malloc(sizeof (struct histo));
  
  // topen opens a file in the appropriate outdir/name directory


  sprintf(h->name, name);
  h->data  = tdata; 
  h->num   = (int)num;
  h->min   = (double) min;
  h->max   = (double) max;
  h->delta = delta;
  h->tot   = 0;       
  h->avg   = 0.0;
  h->var   = 0.0;
  h->val   = 0.0;
  h->maxval = -1e99;
  h->minval = 1e99;
  h->_pseudo_variance = 0.0;
  h->_old_avg = 0.0;
 
  histo_array[histoNum] = h;   
  return histoNum++;
}



/* internal */

void __histo_add(struct histo *h, double val, int dbg) { 
int i;

  h->tot++;
  h->val = val;
  h->maxval = val > h->maxval ? val : h->maxval;
  h->minval = val < h->minval ? val : h->minval;
  h->avg += (val - h->_old_avg) / h->tot;
  h->_pseudo_variance += (val - h->_old_avg) * (val - h->avg);
  if (h->tot > 1) {
    h->var      = h->_pseudo_variance / (h->tot -1);
    // h->stdev    = sqrt(h->var);
    h->_old_avg = h->avg;
  }

  i = (val < h->min)  ? 0 :
      (val >= h->max) ? (int) h->num -1 : 
                        (int) ((val + h->delta - h->min) / h->delta);

//   if (dbg && (!h->tot ||  h->data[(int)h->num-1]  || h->data[0])) {
//      printf("[%d]%f\t[%d]%f\t[%d]%f   ==%d\n", 
//      		h->data[0], h->min, 
// 		i, h->val, 
// 		h->data[(int)h->num-1], h->max, h->tot);
//      }



  if ( i > h->num-1 ) {
     printf("wrong index %d >= %d\n",i,h->num);
  } else {  
     h->data[i]++;
  } 
}

/*
char * histo_info(Histo *h) {   			    
  // sprintf(histo_buf,"%f %f %f", h->val, h->avg, h->stdev);
  sprintf(histo_buf,"%f %f %f", h->val, h->avg, sqrt(h->var));
  return histo_buf;
}
*/


FILE *table = NULL;
int histo_dump_all(void) {
  // topen opens a file in the appropriate outdir/name directory
  if ( histoNum ) {
     sprintf(buf,"column -t > %s/result.%s.TABLE", prefix, postfix);
     if ((table = popen(buf,"w")) == NULL) {
	fprintf (stderr,"Could not open file %s\n", buf);
	exit(1);
     }
     fprintf(table, "#X avg stdev min max prc25 prc50 prc75 prc95 prc99\n");
  }
    
  while (--histoNum>-1) 
	histo_dump(histoNum,1);

  if( table ) fclose(table);	
  return 1;	
}


double perc_thresh[9]={1.0,5.0,10.0,25.0,50.0,75.0,90.0,95.0,99.0};
double perc[9]={0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0};
int __histo_dump(struct histo *h, int print_cdf) {
double cdf = 0.0, oldcdf = 0.0;
int i, perc_id;

  if(h->tot==0) {
     fprintf(stderr,"No data for %s: %d\n", 
     		h->name, h->tot);
	return;	
  }
  	
  sprintf(buf,"%s/%s.%s.cdf", prefix, h->name, postfix);
  if ((h->fp = fopen(buf,"w")) == NULL) {
     fprintf (stderr,"Could not open file %s\n", buf);
     exit(1);
  }
 #ifdef DBG_HISTO
  fprintf(h->fp,"#(%s) min:max:delta:num   %g:%g:%g:%d\n", 
      name, h->min, h->max, h->delta, (int) h->num);
  #endif
 
  if ( print_cdf ) { 
     fprintf(h->fp,"#stats at the end\n#x pdf CDF 1-CDF count{x}\n");
  }

  for (perc_id=0; perc_id<7; perc_id++)
      perc[perc_id]=0;
  
  /* how many rows? +2 to include (premin e postmax) */
  for (i = 0; i < h->num; i++) {
    cdf += (double)h->data[i]/(double)h->tot;
    for (perc_id=0; perc_id<7; perc_id++) {
       if( 100*cdf    >= perc_thresh[perc_id] && 
           100*oldcdf <  perc_thresh[perc_id] ) 
   	      perc[perc_id] = h->min + (i * h->delta) - h->delta;	
    }       
    oldcdf = cdf;
    if ( print_cdf ) {
         double pdf = h->data[i]/(double)h->tot;
	 if(h->data[i])
         fprintf (h->fp, "%g %g %g %g %d\n", 
	    h->min + (i * h->delta) - h->delta, pdf, cdf, 1-cdf, h->data[i]); 
    }	    
  }
  // all results in a table
  fprintf(table, "%s %f %f %f %f %f %f %f %f %f\n" , 
  	h->name, h->avg, sqrt(h->var), h->min, h->max,
	perc[3],perc[4],perc[5],perc[7],perc[8]);
  
  
  // statistics
  fprintf(h->fp,"#min:   %le\n", h->minval);
  fprintf(h->fp,"#max:   %le\n", h->maxval);
  fprintf(h->fp,"#nsmpl: %le\n", (double)h->tot);
  fprintf(h->fp,"#delta: %le\n", h->delta);
  fprintf(h->fp,"#avg:   %le\n", h->avg);
  fprintf(h->fp,"#var:   %le\n", h->var);
  fprintf(h->fp,"#stdev: %le\n", sqrt(h->var));
  for (perc_id=0; perc_id<7; perc_id++)
	fprintf(h->fp,"#%.0fprc: %le\n", perc_thresh[perc_id],perc[perc_id]);
  fclose (h->fp);
  return 1;
}

	
