/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/
/* Revision Control System LOG entries */
/* $Log: stathead.h,v $
 * Revision 1.1  1999/08/19 16:28:39  giaccone
 * Initial revision
 * */

/*
**  Module   : stathead.h
**  Version  : 1.0
**  Author   : Andrea Francini
**  Date     : 17.03.1994
**  Synopsis : This module contains the constant and type definitions
**             and the function prototypes shared by the statistical module 
**             and the statistical interface module.
**             It must be included by "stat_mod.c" and by the header file 
**             (or the statistical interface module) of the whole simulation 
**             program.
**
**
**  Warning  : a set of functions whose prototypes are declared into this 
**             file refers to variables defined into "stat_mod.c".
**             These variables deal with the statistical parameters whose 
**             setting is needed to drive the statistics evaluations.
**
**  The description of these parameters follows (the "stat_mod.c" variables 
**  names are used):
**
**            1. ref_accuracy (default value: 0.05)  
**              -- Relative half-width of the confidence interval required 
**              -- to stop the simulation run (the run is stopped when all 
**              -- the analyzed processes achieve the selected accuracy level).
**
**            2. min_transient_length (default value: 5000)
**              -- Minimum number of observations to be collected before to
**              -- check the initial transient closing condition.
**
**            3. max_coll_obs (default value: 100000)
**              -- Maximum number of treatable observations (the collection 
**              -- of 'max_coll_obs' observations has equivalent effects 
**              -- compared to the achievement of the selected accuracy level).
**
**            4. tr_bat_size (default value: 30)
**              -- Size of a batch used to detect the initial transient 
**              -- duration. 
**
**            5. tr_seq_len (default value: 3)
**              -- Number of transient batch means used to check the 
**              -- initial transient closing condition.
**
**            6. tr_accuracy (default value: 0.99)
**              -- Accuracy level required to close the initial transient 
**              -- detection.
**
**            7. st_bat_size (default value: 10000)
**              -- Initial size of a batch used during the stationary 
**              -- analysis. If the number of collected batch means exceeds 
**              -- the available dedicated memory space, the means are 
**              -- coupled and the batch size doubles.
**
**            8. confid_lev (default_value: 2 ( => 95% ))
**              -- The coupling between the available confidence levels and 
**              -- the respective index values follows:
**                      0       ->      80%
**                      1       ->      90%
**                      2       ->      95%
**                      3       ->      98%
**                      4       ->      99%
**                      5       ->      99.9%   
**
**            These values may be changed by selecting new default values 
**            or by requesting new user values.
**
**
**
**              WARNING!        WARNING!        WARNING!        WARNING!        
**
**              -- The selection of the parameters dealing with the initial 
**              -- transient detection is very critical. If the user has 
**              -- not a thorough deep confidence with this matter the 
**              -- following parameters sequences are suggested:
**
**              a. low load systems:    1. tr_bat_size       : 30
**                                      2. tr_seq_len        : 3
**                                      3. tr_accuracy       : 0.25
**                                      4. st_bat_size : 5000
**
**              b. mean load systems:   1. tr_bat_size       : 30
**                                      2. tr_seq_len        : 10
**                                      3. tr_accuracy       : 0.25
**                                      4. st_bat_size : 10000
**
**              a. high load systems:   1. tr_bat_size       : 30
**                                      2. tr_seq_len        : 15
**                                      3. tr_accuracy       : 0.25
**                                      4. st_bat_size : 20000
**
**              -- To avoid a risky usage of the statistical routines an 
**              -- unprepared user can select the following values (see 
**              -- default values list):
**              
**                      1. min_transient_length : 5000
**                      2. tr_bat_size          : 30
**                      3. tr_seq_len           : 3
**                      4. tr_accuracy          : 0.99
**                      5. st_bat_size    : 10000
**              
**              -- (the values of 'accuracy', 'max_coll_obs' and 'confid_lev' 
**              -- are load level independent).
**                                      
*/


/*
**  Constant Definitions
*/

#define STAT_HDR

#ifndef TRUE
#define TRUE        (unsigned char)1
#endif

#ifndef FALSE
#define FALSE       (unsigned char)0
#endif

#define MAX_STAT_VAR    30     /* Maximum number of stochastic processes     */
                               /* treatable by the statistical methods.      */

#define NPRCNTL  6             /* Number of percentiles available with an    */
                               /* assigned number of freedom degrees.        */

#define INT_UNCHANGED    (int)     -1   /* Constant value for default        */
                                        /* 'int' value assignment            */
                                        /* (see 'set_*()' functions).        */
#define LONG_UNCHANGED   (long)    -1   /* Constant value for default        */
                                        /* 'long' value assignment           */
                                        /* (see 'set_*()' functions).        */
#define DOUBLE_UNCHANGED (double)  -1.0 /* Constant value for default        */
                                        /* 'double' value assignment         */
                                        /* (see 'set_*()' functions).        */


/* 
**  Type Definition
*/

typedef unsigned char       Booltype; /* Local Boolean type.                 */


/*
**  Function Prototypes
*/

/*
** Function: void init_statistics(...)
** Return  : None
** Remarks : this function sets the variables used by the statistical 
**           routines, according to the programmer (or user) needs.
**
** Parameters description:
**
**           1. n_stat_var      : number of variables to apply to the mean and 
**              variance estimation;             
**           2. significant_var : array of boolean flags to recognize the 
**              variables to apply to the complete statistical analysis (if 
**              NULL the analysis applies to all the variables);
**           3. var_names       : array of string pointers; the items referred 
**              to by the pointers are the names of the observed variables 
**              (NULL if automatic names assignement is required);
**           4. end_transient   : array of pointers to the functions to execute 
**              the operations required when the i-th variable transient is 
**              over (NULL if no operations are required);
**           5. end_analysis    : array of pointers to the functions to execute 
**              the operations required when the i-th variable stationary 
**              accuracy is achieved (NULL if no operations are required);
**           6. end_sim_run     : pointer to the function to execute the 
**              operations  required when the simulation run is over (Fatal 
**              Error if NULL); 
**           7. stat_file_p     : pointer to the output file to write onto 
**              the error messages and the statistical report (if NULL, the 
**              file name assignment becomes automatic);
*/
void init_statistics(
                     int n_stat_var,
                     Booltype significant_var[], 
                     char *var_names[],
                     void (*end_transient[])(), 
                     void (*end_analysis[])(),
                     void (*end_sim_run)(), 
                     FILE *stat_file_p
                    );


/*
** Function : void d_statistics(int var_id, double var_val)
** Return   : None
** Remarks  : externally usable function to lead the collection and analysis
**            of discrete-time statistical variables.
**
** Parameters description:
**
**            1. var_id : variable identifier;
**            2. var_val: variable value.
*/
void d_statistics(int var_id, double var_val);


/*
** Function : void c_statistics(int var_id, double var_val, double sim_time)
** Return   : None
** Remarks  : externally usable function to lead the collection and analysis
**            of continous-time statistical variables.
**
** Parameters description:
**
**            1. var_id   : variable identifier;
**            2. var_val  : variable value;
**            3. sim_time : current simulation time.
*/
void c_statistics(int var_id, double var_val, double sim_time);


/*
** Function : void b_statistics(int var_id, int var_val)
** Return   : None
** Remarks  : externally usable function to lead the collection and analysis
**            of binary statistical variables.
**
** Parameters description:
**
**            1. var_id : variable identifier;
**            2. var_val: variable value (0 or 1).
*/
void b_statistics(int var_id, int var_val);


/*
** Function: void report_statistics(FILE *fpout, int var_id, 
**                                  Booltype still_in_transient)
** Return  : None
** Remarks : this function writes onto the file pointed by 'fpout' 
**           the statistical report referring to the variable indexed 
**           by 'var_id'.
**           
**           still_in_transient = TRUE : end of transient report is required;
**           still_in_transient = FALSE: stationary report is required;
*/
void report_statistics(FILE *fpout, int var_id, Booltype still_in_transient);


/*
** Function: Booltype transient_is_over(int var_id)
** Return  : TRUE  if the initial transient is over for the variable indexed 
**	     	   by 'var_id';
**	     FALSE if the initial transient is not over for the variable 
**	     	   indexed by 'var_id'.
*/
Booltype transient_is_over(int var_id);


/*
** Function: long curr_nr_of_samples(int var_id)
** Return  : the number of observations collected during the current 
**	     simulation phase (transient or stationary state) for the 
**	     variable indexed by 'var_id'.
*/
long curr_nr_of_samples(int var_id);


/*
** Function: double curr_mean(int var_id)
** Return  : the current stationary mean value referring to the variable 
**           indexed by 'var_id'.
** Remarks : the returned value is the most recently evaluated available 
**           value.
*/
double curr_mean(int var_id);


/*
** Function: double curr_variance(int var_id)
** Return  : the current stationary variance value referring to the variable 
**           indexed by 'var_id'.
** Remarks : the returned value is the most recently evaluated available 
**           value.
*/
double curr_variance(int var_id);


/*
** Function: double curr_accuracy(int var_id)
** Return  : the current stationary accuracy value referring to the variable 
**           indexed by 'var_id'.
** Remarks : the returned value is the most recently evaluated available 
**           value.
*/
double curr_accuracy(int var_id);


/*
** Function: void set_ref_accuracy(double value)
** Return  : None
** Remarks : this function may be called to set 'ref_accuracy' value.
**           'ref_accuracy' is a "stat_mod.c" reserved variable.
**
**           'value' is the new 'ref_accuracy' value.
**           Legal range of 'value' : (0.0,1.0) .
**
**           Default value of 'ref_accuracy' :  0.05 
**
**           IF         (value = DOUBLE_UNCHANGED)
**           THEN       the default value is assigned to 'ref_accuracy'
**           ENDIF
**      
**           IF         (value is out of range)
**           THEN       an error message appears into the proper file and 
**                      the default value is assigned to 'ref_accuracy'
**           ELSE       'value' is assigned to 'ref_accuracy'
**           ENDIF
**
**
**           WARNING !! - The call to this function must mandatorily 
**                      - follow the call to 'init_statistics(...)'.
**          
**           
*/
void set_ref_accuracy(double value);


/*
** Function: void set_confid_lev(int index)
** Return  : None
** Remarks : this function may be called to set 'confid_lev' value.
**           'confid_lev' is a "stat_mod.c" reserved variable.
**
**           'index' is the new 'confid_lev' value.
**           Legal range of 'index' : [0,NPRCNTL-1] .
**
**           Default value of 'confid_lev' :  2 
**
**           IF         (index = INT_UNCHANGED)
**           THEN       the default value is assigned to 'confid_lev'
**           ENDIF
**      
**           IF         (value is out of range)
**           THEN       an error message appears into the proper file and 
**                      the default value is assigned to 'confid_lev'
**           ELSE       'index' is assigned to 'confid_lev'
**           ENDIF
**
**
**           WARNING !! - The call to this function must mandatorily 
**                      - follow the call to 'init_statistics(...)'.
**           
*/
void set_confid_lev(int index);


/*
** Function: void set_max_coll_obs(long noobs)
** Return  : None
** Remarks : this function may be called to set 'max_coll_obs' value.
**           'max_coll_obs' is a "stat_mod.c" reserved variable.
**
**           'noobs' is the new 'max_coll_obs' value.
**           Legal range of 'noobs' : noobs > 0 .
**
**           Default value of 'max_coll_obs' :  100000 
**
**           IF         (noobs = LONG_UNCHANGED)
**           THEN       the default value is assigned to 'max_coll_obs'
**           ENDIF
**      
**           IF         (value is out of range)
**           THEN       an error message appears into the proper file and 
**                      the default value is assigned to 'max_coll_obs'
**           ELSE       'noobs' is assigned to 'max_coll_obs'
**           ENDIF
**
**
**           WARNING !! - The call to this function must mandatorily 
**                      - follow the call to 'init_statistics(...)'.
**           
*/
void set_max_coll_obs(long noobs);

/* This function has been added by Paolo Giaccone 12/14/01 */
void set_min_coll_obs(long noobs);



/*
** Function: void set_min_transient_length(long value)
** Return  : None
** Remarks : this function may be called to set 'min_transient_length' value.
**           'min_transient_length' is a "stat_mod.c" reserved variable.
**
**           'value' is the new 'min_transient_length' value.
**           Legal range of 'value' : value >= 0 .
**
**           Default value of 'value' :  5000 
**
**           IF         (value = LONG_UNCHANGED)
**           THEN       the default value is assigned to 
**                      'min_transient_length'. 
**           ENDIF
**      
**           IF         (value is out of range)
**           THEN       an error message appears into the proper file and 
**                      the default value is assigned to 
**                      'min_transient_length' 
**           ELSE       'value' is assigned to 'min_transient_length'
**           ENDIF
**
**
**           WARNING !! - The call to this function must mandatorily 
**                      - follow the call to 'init_statistics(...)'.
**           
*/
void set_min_transient_length(long value);


/*
** Function: void set_tr_bat_size(long value)
** Return  : None
** Remarks : this function may be called to set 'tr_bat_size' value.
**           'tr_bat_size' is a "stat_mod.c" reserved variable.
**
**           'value' is the new 'tr_bat_size' value.
**           Legal range of 'value' : value > 0 .
**
**           Default value of 'value' :  30 
**
**           IF         (value = LONG_UNCHANGED)
**           THEN       the default value is assigned to 'tr_bat_size'
**           ENDIF
**      
**           IF         (value is out of range)
**           THEN       an error message appears into the proper file and 
**                      the default value is assigned to 'tr_bat_size'
**           ELSE       'value' is assigned to 'tr_bat_size'
**           ENDIF
**
**
**           WARNING !! - The call to this function must mandatorily 
**                      - follow the call to 'init_statistics(...)'.
**           
*/
void set_tr_bat_size(long value);


/*
** Function: void set_tr_seq_len(long value)
** Return  : None
** Remarks : this function may be called to set 'tr_seq_len' value.
**           'tr_seq_len' is a "stat_mod.c" reserved variable.
**
**           'value' is the new 'tr_seq_len' value.
**           Legal range of 'value' : value >= 2 .
**
**           Default value of 'value' :  3 
**
**
**
**	     WARNING !!
**
**	     IF 	(value = 0)
**	     THEN	the initial transient detection is not carried out.
**	     ENDIF
**
**
**           IF         (value = LONG_UNCHANGED)
**           THEN       the default value is assigned to 'tr_seq_len'
**           ENDIF
**
**
**           IF         (value is out of range)
**           THEN       an error message appears into the proper file and 
**                      the default value is assigned to 'tr_seq_len'
**           ELSE       'value' is assigned to 'tr_seq_len'
**           ENDIF
**
**
**           WARNING !! - The call to this function must mandatorily 
**                      - follow the call to 'init_statistics(...)'.
**           
*/
void set_tr_seq_len(long value);


/*
** Function: void set_tr_accuracy(double value)
** Return  : None
** Remarks : this function may be called to set 'tr_accuracy' value.
**           'tr_accuracy' is a "stat_mod.c" reserved variable.
**
**           'value' is the new 'tr_accuracy' value.
**           Legal range of 'value' : (0.0,1.0) .
**
**           Default value of 'tr_accuracy' :  0.99 
**
**           IF         (value = DOUBLE_UNCHANGED)
**           THEN       the default value is assigned to 'tr_accuracy'
**           ENDIF
**      
**           IF         (value is out of range)
**           THEN       an error message appears into the proper file and 
**                      the default value is assigned to 'tr_accuracy'
**           ELSE       'value' is assigned to 'tr_accuracy'
**           ENDIF
**
**
**           WARNING !! - The call to this function must mandatorily 
**                      - follow the call to 'init_statistics(...)'.
**          
**           
*/
void set_tr_accuracy(double value);


/*
** Function: void set_st_bat_size(long value)
** Return  : None
** Remarks : this function may be called to set 'st_bat_size' value.
**           'st_bat_size' is a "stat_mod.c" reserved variable.
**
**           'value' is the new 'st_bat_size' value.
**           Legal range of 'value' : value >= 2 .
**
**           Default value of 'value' :  10000 
**
**           IF         (value = LONG_UNCHANGED)
**           THEN       the default value is assigned to 
**                      'st_bat_size'
**           ENDIF
**      
**           IF         (value is out of range)
**           THEN       an error message appears into the proper file and 
**                      the default value is assigned to 'st_bat_size'
**           ELSE       'value' is assigned to 'st_bat_size'
**           ENDIF
**
**
**           WARNING !! - The call to this function must mandatorily 
**                      - follow the call to 'init_statistics(...)'.
**           
*/
void set_st_bat_size(long value);

