/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>

#include "incl.h"

extern int errno;

extern double uniform(double,double,long *);
extern double uniform01(long *);
extern double trunc_exp(double,long,long *);
extern double negexp(double,long *);
extern double normal(double,double,long *);

void schedule(int type, double time, EventParam par)
{
   Event *ev;
   ev = new_event();
   ev->type = type;
   ev->time = time;
   ev->par=par;
   insert_event(&event_list,ev);
   return;
}


void print_usage() {
printf( "\nwow-%.2f (%s flavor)\n", Version, Flavor);
printf( "\nusage:"
	"\nwow [general opt] [traffic opt] [channel opt] [algorithm opt]"
	"\n"
	"\nwhere:"
	"\n----[general options]---------------------------------------"
	"\n    -D: debug flag - a debug file (<debug.out>) is created in "
	"\n	   the directory <debug>"   
	"\n"  
	"\n    -?: help - the available options are displayed"  
	"\n    -T: test mode (use all defaults values, useful to sneak SEGV)" 
	"\n    -f: force to overwrite existing files"
	"\n    -O: use output <dir> rather than default one"
	"\n    -C: quit after <num> cycles" 
	"\n	   by default, simulation terminates when networking" 
	"\n	   performance converges; however, to study traffic" 
	"\n	   dynamic, a lower cycles number may be sufficient"
	"\n"
	"\n----[traffic model options]---------------------------------------"
	"\n   -t: traffic model {0:Poisson, 1:NaSch,	2:NaSch-(muSTEP),"
	"\n				     3:Gipps(*),4:Gipps-(muSTEP),"
	"\n				     4:VDR,	6:VDR-(muSTEP),"
	"\n				     7:Tsquare, 8:Tsquare-(muSTEP),"
	"\n				     9:Fukui,  10:Fukui-(muSTEP),"
	"\n				     11:TOCA}"
	"\n    -R: [realistic models] vehicles'density [vehs/km]"	       
	"\n    -l: [Poisson] vehicles'arrival rate "	       
	"\n    -v: [Poisson] speed pdf {unif|cost|exp}"	       
        "\n    -n: [Poisson] new cars arrival period after the accident(min)"
        "\n    -s: [Poisson] time between consecutive broadcast cycles (min)"
        "\n"				        		       
	"\n----[channel model options]---------------------------------------"
	"\n    -c: channel model {0:ErrorFree, 1:Bernoulli, 2:Transitional, 3:Gilbert}"
	"\n    -x: parameter <x> meaning depend from the channel_model:"        
	"\n	      CHANNEL_BERNOULLI: parameter <x> = loss probability"
	"\n	      CHANNEL_TRANSITIONAL: parameter <x> = channel.transition"				   
	"\n"
	"\n----[algorithmic options]-----------------------------------------"
	"\n    -a: algorithm (none|optimum|pflood|d2flood|help|hybrid|d3flood)"    
	"\n    -r: vehicles transmission range (m)"
	"\n    -P: [pflood]  flooding probability"
	"\n    -k: [d2flood] k power"
	"\n    -2: [d2flood] d2flood rule number"
	"\n    -h: [d3flood] h power"
	"\n    -3: [d2flood] d3flood rule number"
	"\n    -B: [help]    Average Beacon Transmission Interval (sec)"        
	"\n    -e: [help]    <epsilon> fixed error margin (m)"
	"\n    -E: [help]    rule number for adaptive margin"
	"\n    -y: parameter <y> meaning depend from the algorithmic context:"
	"\n        ALGO HYBRID: hybridation type (see help.c)"
	"\n    -H: [hybrid]  Hybridity degree in [0,1], with 0=>help"	       
	"\n     	     use -y to tune behavior (see help.c)"      
	"\n"    
	"\n\nnote:"
	"\n	at least one arg should be specified; for all the others,"
	"\n	their default value will be selected "
	"\nIMPORTANT:"
	"\n	directory debug must exist (even in non-debug mode)"
	"\n      otherwise SEGV will be raised"
	"\n\n\n");
	exit(-1);
}

extern char* optarg;
void input_parameters(int argc, char** argv) {
  char ch, filename[1024];  
  int rate,i;  
   
  MAX_CYCLES= 100000000;
  MAX_CYCLES= 20000;

  channel.model    = CHANNEL_ERROR_FREE;
  channel.one_shot = FALSE;
  channel.markovian= FALSE;
  channel.gilbert.rate[CHANNEL_GOOD] = 0.5;
  channel.gilbert.rate[CHANNEL_BAD]  = 0.5;


  // probabilistic flooding, by default
  sprintf(algostr,"pflood"); 
  algo = ALGO_PFLOOD;
  Pflood=	1;     // -p
  maximum_time= 1e12;  // -t
  range=	200;   // -r
  uiDebugMask=	0;     // -D (flag)
  min=		0;     // -m
  AI=		0;     // -s
  d2flood_k=	8;     // for d^2-flooding algorithm (ASWN'06)
  d3flood_h=	2;     // *** 
  d2flood_rule=	0;     // *** new stuff I'm working on  
  d3flood_rule=	0;     // *** 
  HelloFreq=1;	       // for HELP algorithm (MP2P'06)
  Hybrid=0.5;	       // (h)HELP + (1-h)d2flood @ AUTONET/GLOBECOM'06
  epsilon=0.0;
  sprintf(prefix,"./");
  system_rho=20;

  // default for TRAFFIC_NASCH
  traffic.cell=7.5;   //(m)
  traffic.step=1.2;   //(s)
  traffic.vmax=5;     //(cell/step)
  traffic.pdec=0.16; //
  traffic.vunit=1;    //	
  traffic.dunit=1;    //
  
  // for VDR, FUKUI and TSQ
  for (i=0; i<=(int)traffic.vmax; i++)
     traffic.vdr.pdec[i]=i/5*(2*traffic.pdec);

  for (i=0; i<3; i++)
     traffic.vdr.pdec[i]=0.11;
  for (i=3; i<6; i++)
     traffic.vdr.pdec[i]=0.21;

  // for TSQ
  traffic.tsq.incr_pdec = 0.5;
  
  // default for TRAFFIC_GIPPS
  // from 0.7 on, Gipps is the default model
  traffic.model = TRAFFIC_GIPPS;
  traffic.gipps.a=0.2; // cell/(step^2)
  traffic.gipps.b=0.6; // cell/(step^2)
  traffic.gipps.epsilon=1.0; // in [0,1]
  STREET_LENGTH=800*traffic.cell;
  SecurityArea=267*traffic.cell;
  MP=STREET_LENGTH;

  
  // default for POISSON (NBNA)
  sprintf(speed,"%s","cost");
  lambda=1/ONE_SECOND;
  if (argc<2)
  	print_usage();

  while ((ch = getopt(argc, argv, "fx:y:a:c:e:E:B:r:n:s:k:h:2:3:P:R:l:v:DTC:O:?t:H:")) != EOF) {
    switch(ch) {
	case 'D': uiDebugMask=-1; 		break;
	case 'f': use_force=1; 			break;
	case 'C': MAX_CYCLES=atoi(optarg);	break;
	case 'x': par_x=atof(optarg);	        break;
	case 'y': par_y=atoi(optarg);	        break;
        case 'e': epsilon= atof(optarg);        break;		
        case 'E': AdaptiveMargin=atoi(optarg);  break;		
        case 'B': HelloFreq= atof(optarg);      break;		
        case 'H': Hybrid = atof(optarg);        break;		
	case 'r': range=atoi(optarg);	        break;
	case 'n': min=atof(optarg);	        break;
	case 's': AI=atof(optarg);	        break;
        case 'k': d2flood_k = atoi(optarg);     break; 		
        case '2': d2flood_rule = atoi(optarg);  break; 		
        case 'h': d3flood_h = atoi(optarg);     break; 		
        case '3': d3flood_rule = atoi(optarg);  break; 		
	case 'P': Pflood=atof(optarg);        	break;
	case 'R': system_rho=atof(optarg);      break; 	
	case 'l': lambda=atof(optarg)/ONE_SECOND;          break; 	
	case 'v': sprintf(speed,"%s",optarg);   break; 	
       
        case 'O': 
            sprintf(prefix,"%s",optarg);
	    //if (index(prefix, '/')) {
        	if (stat (prefix, &fbuf) == -1) {
        	   fprintf (stderr,"Creating output dir <%s>\n", prefix);
        	   mkdir (prefix, 0775);
        	}
	    //} 	
	    break; 		
        case 'T': 
            printf("Test-mode\n");
	    break; 		
	    
        case 'a': sprintf(algostr,"%s",optarg);    
		  if(!strcmp(algostr,"none")) {
		  	// no networking, just traffic analysis
		  	MAX_CYCLES=666;	
			algo = ALGO_NONE;
			
		  } else if (!strcmp(algostr,"optimum")){
			algo = ALGO_COVERAGE;
			
 	          } else if (!strcmp(algostr,"d2flood")){
			algo = ALGO_D2FLOOD;

 	          } else if (!strcmp(algostr,"d3flood")){
			algo = ALGO_D3FLOOD;

 	          } else if (!strcmp(algostr,"help")){
			algo = ALGO_HELP;
		  
 	          } else if (!strcmp(algostr,"hybrid")){
			algo = ALGO_HYBRID; // new from v0.7+
		  
 	          } else if (!strcmp(algostr,"pflood")){
			algo = ALGO_PFLOOD;
		  
		  } else {
			fprintf(stderr,"Unknown algorithm ``%s''\n",optarg);
			fprintf(stderr,"Valid algorithms are (none|optimum|pflood|d2flood|help|hybrid|d3flood)");			
			exit(-1);	
		  }
		  fprintf(stderr,"Using ``%s'' algorithm (ID=%d)\n",algostr,algo);
		  
		  break;		
		  
	
	case 'c': /* channel model */
		  channel.model = atoi(optarg);
		  switch(channel.model) {
			case CHANNEL_ERROR_FREE:
			        break;
		        case CHANNEL_BERNOULLI:   				
				channel.perror = par_x;
				break;
				
		        case CHANNEL_TRANSITIONAL:   				
				channel.transition = par_x;
		  		fprintf(stderr,"CHANNNEL %f\n", channel.transition);
				break;
				
		        case CHANNEL_GILBERT:   				
				channel.perror = channel.gilbert.rate[CHANNEL_GOOD]/(channel.gilbert.rate[CHANNEL_GOOD]+channel.gilbert.rate[CHANNEL_BAD]);
				break;
					
			default:				
				fprintf(stderr,"Unknown channel model %d\n",atoi(optarg));
				exit(1);
		  }	
		  fprintf(stderr,"Using ``%s'' channel model\n",
		  	channel_name[channel.model]);
		  break;
		  

	case 't': /* traffic model */
		  traffic.model = atoi(optarg);
		  switch(traffic.model) {
			case TRAFFIC_NBNA:		 
				STREET_LENGTH=2000.0;
				SecurityArea=2000;
  				MP=STREET_LENGTH;
				break;
			case TRAFFIC_NASCH:	
			case TRAFFIC_VDR:	
			case TRAFFIC_TSQ:	
			case TRAFFIC_FUKUI:		
			case TRAFFIC_TOCA:		
				break;

			case TRAFFIC_GIPPS:	
				traffic.step=1.0;  //(s)
				break;
				
			case TRAFFIC_NASCH_MINISTEP:   				
			case TRAFFIC_VDR_MINISTEP:	
			case TRAFFIC_TSQ_MINISTEP:	 
			case TRAFFIC_FUKUI_MINISTEP: 
				fprintf(stderr,"NaSch CA Traffic Model (MINI_STEP)\n");
				traffic.step=0.12;  //(s)
				traffic.vmax=0.5;   //(cell/step)
				traffic.vunit=0.1;  //
				break;
				
			case TRAFFIC_GIPPS_MINISTEP:   
				traffic.step=0.12;   //(s)
				traffic.vmax=0.5;    //(cell/step)
				traffic.gipps.a=0.1*(traffic.step*traffic.step); // cell/(step^2)
				traffic.gipps.b=0.6*(traffic.step*traffic.step); // cell/(step^2)

				break;
			default:				
				fprintf(stderr,"Unknown traffic model %d\n",atoi(optarg));
				exit(-1);
		  }	
		  fprintf(stderr,"Using ``%s%s'' traffic model\n",
		  	traffic_name[traffic.model],
			(traffic.model && !(traffic.model%2)) ? " (muSTEP)" : "");		  
		  break;

    	case '?':
	default: print_usage(); 
		 break;
     }     
  }   	
  
 /* JSAC? */ 
 Pflood_constant = Pflood;

 txRate=2e6/1.0e6;            // bit/mus
 txTime=1000.0*8/txRate;    // 1000 Bytes packets
 d2flood_win= txTime/SlotTime;
 
 txTimeHello=sizeHello*8/txRate;  
 rxTime=txTime;             // 1000 Bytes packets
 rate=txRate/10.0e5;

 if (channel.model == CHANNEL_BERNOULLI) 
 	channel.perror = par_x;

 H_numw=-1;
 H_1D=-1;
 H_09D=-1;
 H_05D=-1;
 H_025D=-1; 
 H_alfa=-1;
 H_dRelay=-1;
 H_vicini=-1;
 H_neighbors=-1; 
 H_speed=-1; 

 
 if(algo==ALGO_PFLOOD)
    fprintf(stderr,"P_flood   = %f\n",Pflood);
 fprintf(stderr,"Max Time  = %f\n",maximum_time);		
 fprintf(stderr,"Max Dist  = %f\n",STREET_LENGTH);		
 fprintf(stderr,"Tx Range  = %ld\n",range);			
 fprintf(stderr,"Tx Rate   = %e\n",txRate);			
 fprintf(stderr,"Tx Time   = %f\n",txTime);
 fprintf(stderr,"Traffic Model   = %s\n",traffic_name[traffic.model]);
 if(traffic.model > TRAFFIC_NBNA){
    fprintf(stderr,"Traffic  Density = %f\n",system_rho);
 }else{
    fprintf(stderr,"Arrival rate = %f\n",lambda*ONE_SECOND);
 }


 //===================================================================================
// choose prefix/postfix
//-----------------------------------------------------------------------------------
     char algodescr[128];
     switch(algo) {
        case ALGO_PFLOOD:
     		sprintf(algodescr, "%s%.0f", algostr, 100.0*Pflood);
	    	break;
        case ALGO_D2FLOOD:
     		sprintf(algodescr, "%s%dK%dM", algostr, d2flood_k, d2flood_rule);
	    	break;
        case ALGO_D3FLOOD:
     		sprintf(algodescr, "%s%dK%dM%dH%dM", algostr, 
				d2flood_k, d2flood_rule,				
				d3flood_h, d3flood_rule);	
	    	break;
        case ALGO_HELP:
     		sprintf(algodescr, "%sB%.0fe%.0fE%d", algostr, 
			HelloFreq, epsilon, AdaptiveMargin);
	    	break;   	
        case ALGO_HYBRID:
     		sprintf(algodescr, "%sH%.0fK%dB%.0fe%.0fE%d.%s", algostr, 
		        Hybrid*100.0,  
			d2flood_k,
			HelloFreq, 
			epsilon, 
			AdaptiveMargin,
			hybrid_nickname[par_y]);
	    	break;   	
	default:
	     	sprintf(algodescr, algostr);
     }	
     switch(channel.model) {
        case CHANNEL_TRANSITIONAL:	
        case CHANNEL_BERNOULLI:	
		if(par_x > 0.5) {
		    printf("Setting a loss probability of %f is not a good idea\n",par_x);
		    exit(1); 
		}
	     	sprintf(algodescr, "%s.%s%.0f", algodescr, 
			channel_nickname[channel.model], par_x*100.0);
	    	break;
     	
	default:
	     	sprintf(algodescr, "%s.%s", algodescr, channel_nickname[channel.model]);

     }

     if(traffic.model > TRAFFIC_NBNA){
        sprintf(postfix,"%s.%s.R%.0f",traffic_nickname[traffic.model], algodescr, system_rho);
     }else{
        sprintf(postfix,"%s.%s.l%.1f",traffic_nickname[traffic.model], algodescr, lambda*ONE_SECOND);
     }
     printf("%s\n",postfix);

 histo_setup(prefix,postfix);
 sprintf(filename,"%s/res.%s",prefix,postfix);
 
 
 if ((fileout = fopen (filename, "r")) != NULL) {
     if(!use_force) {
         printf("Preventing overwrite of %s\n",filename);
         printf("use option -f to overwrite existing files\n");
         exit(1);
     } else {	  
         printf("Overwriting %s\n",filename);
     }	 
}
 
 
 if ((fileout = fopen (filename, "w")) == NULL) {
   printf ("Error in opening file %s\n", filename);
   printf("%s\n",strerror(errno));
   exit (1);
 }
 fflush(NULL);

 if (channel.model != CHANNEL_ERROR_FREE)
	H_channel   = histo_newD("channel", 0.0, 1.0, 1.0);


  if (algo == ALGO_NONE) {
     H_headw     =  histo_newD("headw", 0.0, 25.0,    0.1);
     H_distance  =  histo_newD("dist",  0.0, 500.0,   0.5);
     H_speed     =  histo_newD("speed", 0.0, 200.0,   0.5);
     H_rho       =  histo_newD("rho",  	0.0, MAX_RHO, 1.0);
     H_density   =  histo_newD("tr.dens",      0.0, 100.0,   0.5);
     H_flow      =  histo_newD("tr.flow",      0.0, 2500.0,  5.0);
     H_velocity  =  histo_newD("tr.velocity",  0.0, 200.0,   0.5);
  }
  
  H_numw     =  histo_newD("NumW",  0,   1000, 1);
  H_1D       =  histo_newD("D1",    0,   2.5,  1e-4);
  H_alfa     =  histo_newD("alfa",  0,   1,	 0.01);
  H_dRelay   =  histo_newD("dRelay",0,   1000,  1);

     if (1) {
	 sprintf(filename,"%s/fund.%s",prefix,postfix);
	 if ((diaf = fopen (filename, "w")) == NULL) {
	    fprintf (stderr,"Error in opening file %s\n", filename);
	    exit (1);
	 }
	 fprintf(diaf,"#density   flow      velocity Pinfo Warn \n");
	 fprintf(diaf,"#[veh/km]  [veh/hr]  [km/hr]  [-]   [-]\n");
     }

 

 fflush(NULL);
}

void carExit(EventParam par_in)
{
  Car *car;
  car = out_list(&queue);
  //DBG_PL(CarExit,"time:%f car:%d"),current_time,car->id DBG_PR;
    
   if (car->relay==1&&car->warning>1){
     dup_relay++;
   }
   if (car->relay==1&&car->warning>0){
     info_relay++;
     //DBG_PL(CarExit,"time:%f car:%d  warnings:%d dup_relay:%d"),current_time,car->id,car->warning,dup_relay DBG_PR;
   }  
  if (car->collided>0&&car->warning==0)
   collided++;

  --system_cars;

  if (current_time>timeFirstAccident){
       W[car->warning]++;
       if(H_numw>=0){     
          if(car->warning>0)
            histo_add(H_numw, car->warning);
      } 
       if(car->relay)
	 Wrel[car->warning]++;
       C[car->collided]++;
       WC[car->warning][car->collided]++;

 }    
  
  release_car(car);

  return;
}


void car_arrival(int pos)
{ 
  Car *car;
  double prob,delta;
  int i;
  EventParam par;
  
  car = new_car();
  car->arrival = current_time;  
  car->id=++total_cars;

  ++system_cars;

  if(traffic.model == TRAFFIC_NBNA){

      /*a new car is created*/
      //updatePosition(&queue);
      car->position=0.0; 
      car->x=0; 
      if(!strcmp(speed,"unif")){
	  car->v=uniform(speed_min,speed_max,&seed);
      }
      else if(!strcmp(speed,"cost")){
	 car->v=(speed_min+speed_max)/2;
      }
      else if(!strcmp(speed,"exp")){
	 car->v=trunc_exp((speed_min+speed_max)/2*1e6,floor(speed_max*1e6),&seed)/1e6;  
      }
      car->minDist=0.0;
      double exitTime=STREET_LENGTH/car->v+current_time;
      car->exitTime=exitTime;
       /*the car passes the accident ->exits from the system*/
      //DBG_PL(CarArrival,"id:%d exit:%f"),car->id,exitTime DBG_PR;
      schedule(CAR_EXIT,exitTime,par);
      /*next arrival*/
      delta = negexp(1.0/lambda,&seed);
      //DBG_PL(CarArrival,"next arrival:%f"),(current_time+delta)/ONE_SECOND DBG_PR;

      if(current_time+delta < (timeFirstAccident+min*1e6))
	 schedule(CAR_ARRIVAL,current_time+delta,par);

  }else{
     car->v=0;
     car->lastv=-1;
     car->x=pos;
     car->position=car->x*traffic.cell; 
     car->minDist=(double)STREET_LENGTH;
  }
  car->firstWait=1;
  car->warning=0; 
  car->collided=0; 

  car->CAR_State=DRIVE;
  car->pkt_inrx=0;
  car->TxEND=0.0;
  car->rx=0;
  car->last_rx_packet_end = 0;

  car->MyHelloC=0;
  for (i=0;i<MAX_CAR;i++){
   car->vicPos[i]=0;
   car->vicSpeed[i]=0;
   car->myPos[i]=0;
   car->mySpeed[i]=0;
   car->HelloTxTime[i] =0;   
   car->HelloC[i] =0;   
   car->minPos=STREET_LENGTH;   
 }	 
 
  if(algo==ALGO_PFLOOD){
     prob=uniform01(&seed);
     car->Pf=0;
     if(prob<Pflood)
       car->relay=1;
     else
       car->relay=0;
  }
  

  if(current_time > timeFirstAccident){ /*it is a car arrived after the accidend*/
       car->NEW=1;
  }else{     
       car->NEW=0;
  }  
  in_list_car(&queue,car);  
  fast_car_search[car->id] = car;
             
  return;
}



void init_var(){
 int i;

 stat_flag=STAT_OFF;
 current_time = 0.0;
 total_cars=system_cars=0; 
 prec=0; 
 cars_accidentTime=0;
 informed=0;
 dup_relay=0;
 info_relay=0;
 collided=0;
 maxadvised=0,carback=0;
 endxnoone_further=endxcoll=0; 
 firstTx=1;
 totW=0;
 minPos=STREET_LENGTH;
 lastUpdate=0.0;
 HelloCounter=0;
 
 have_1D = 0,
 have_09D = 0, 
 have_05D = 0, 
 have_025D = 0 ;
 

 for (i=0;i<10000;i++)
    vettPos[i]=0;

 for (i=0;i<1000;i++)
   Dfwd[i]=0;
   
#ifdef  FAST_SEARCH
 for (i=0;i<MAX_CAR;i++)
   fast_car_search[i] = NULL;   
#endif
   
 
 if(uiDebugMask == -1){
    fhpDebugFile=fopen("debug/debug.out","wr");
 }
}
