/* 
*  wowSim: Wireless on Wheel Simulator
*
*  COPYRIGHT: 
*  Roberta Fracchia, Dario Rossi (c) 2008 
*
*  LICENSE:    
*  GPL, see LICENSE.txt for details 
*
*  DISCLAIMER: 
*  THE SOFTWARE IS PROVIDED "AS IS".  IN NO EVENT SHALL AUTHORS BE LIABLE TO ANY PARTY 
*  FOR DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST 
*  PROFITS, ARISING OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION. 
* 
*                                               _____________
*                                   ..---:::::::-----------. ::::;;.
*                                .'""""""                  ;;   \  ":.
*                             .''                          ;     \   "\__.
*                           .'                            ;;      ;   \\";
*                         .'                              ;   _____;   \\/
*                       .'                               :; ;"     \ ___:'.
*                     .'--...........................    : =   ____:"    \ \
*                ..-""                               """'  o"""     ;     ; :
*           .--""  .----- ..----...    _.-    --.  ..-"     ;       ;     ; ;
*        .""_-     "--""-----'""    _-"        .-""         ;        ;    .-.
*     .'  .'                      ."         ."              ;       ;   /. |
*    /-./'                      ."          /           _..  ;       ;   ;;;|
*   :  ;-.______               /       _________==.    /_  \ ;       ;   ;;;;
*   ;  / |      """"""""""".---."""""""          :    /" ". |;       ; _; ;;;
*  /"-/  |                /   /                  /   /     ;|;      ;-" | ;';
* :-  :   """----______  /   /              ____.   .  ."'. ;;   .-"..T"   .
* '. "  ___            "":   '""""""""""""""    .   ; ;    ;; ;." ."   '--"
*  ",   __ """  ""---... :- - - - - - - - - ' '  ; ;  ;    ;;"  ."
*   /. ;  """---___                             ;  ; ;     ;|.""
*  :  ":           """----.    .-------.       ;   ; ;     ;:
*   \  '--__               \   \        \     /    | ;     ;;
*    '-..   """"---___      :   .______..\ __/..-""|  ;   ; ;
*        ""--..       """--"        m l s         .   ". . ;
*              ""------...                  ..--""      " :
*                         """"""""""""""""""    \        /
*                                                "------"
* 
*  
*  ASCII-ART: courtesy of 
*  http://www.ascii-art.de/ascii/c/car.txt
*/
#ifndef VAR_H
#define VAR_H

#ifdef DECLARE_VAR
#define DECLARE 
#define INIT_AS(val) =val

#else 

#define DECLARE  extern
#define INIT_AS(val)  
#endif 

 enum Events {CAR_ARRIVAL, CAR_EXIT,UPDATE,ACCIDENT, TX_INIT, RX_INIT, TX_END,\
     RX_END,HELLO,HELLO_TX,HELLO_RX,HELLO_TX_END,HELLO_RX_END,TX_TRYAGAIN,WAIT};

DECLARE double Version  INIT_AS(0.75);
DECLARE char   Flavor[] INIT_AS("ICC'08");

DECLARE Car *fast_car_search[MAX_CAR];

// simulator internals
DECLARE Event 	*free_events;
DECLARE Event 	*event_list;
DECLARE Car 	*queue;
DECLARE Coll 	*collision;

// debug and other features
DECLARE int 	MAX_CYCLES	INIT_AS(1e6); 
DECLARE char 	run_on 		INIT_AS(TRUE); /* flag che indica se continuare o no la simulazione */
DECLARE int 	stat_flag;           /* flag for keeping trace of the system behaviour */
DECLARE FILE 	*fileout;
DECLARE long  	seed;
DECLARE int 	use_force 	INIT_AS(0); // overwrite files ?

DECLARE int 	uiDebugMask	INIT_AS(FALSE);    /*if =-1 debug=ON*/
DECLARE FILE 	*fhpDebugFile, *diaf, *checkf, *failf;

DECLARE double 	maximum_time, 
		current_time, 
		last_event_time;


// mobility model 
DECLARE double 	system_rho;
DECLARE double 	lambda;
DECLARE char 	speed[80];
DECLARE double  speed_min INIT_AS(40.0/3600/1000); /*speed(m/mus): 
                                                  uniform(speed_min,speed_max)*/
DECLARE double  speed_max INIT_AS(140.0/3600/1000); 
DECLARE int 	system_cars, total_cars;
DECLARE int 	vettPos[10000]; 
DECLARE double 	STREET_LENGTH, MP, SecurityArea;
DECLARE int 	poisDistr; // set to 0 unless NaSch
DECLARE int     prec INIT_AS(0);

DECLARE struct {
		int 	model;  //TRAFFIC_xxxxx[_MINISTEP];
	    	int	rhoc;
	    	double  density; 
	    	double 	flow; 
	    	double 	cell; 
	    	double 	v; 
	    	double 	vmax; 
		double  vunit;
		double  dunit;
		double  tunit;
	    	double 	step;
		double  pdec;
		double  epsilon;
		struct  { double a,b,epsilon; } gipps;
		struct  { double pdec[20];    } vdr;
		struct  { double incr_pdec;   } tsq;
	} traffic; 

DECLARE double 	minPos;

#ifndef	 DECLARE_VAR
extern const char* traffic_name[];
extern const char* traffic_nickname[];
#else
const char* traffic_name[] = { 
	       "PoissonNBNA",  
	       "NaSch", 
	       "NaSch", 
	       "Gipps", 
	       "Gipps", 
	       "VDR", 
	       "VDR", 
	       "Tsquare", 
	       "Tsquare", 
	       "Fukui-Ishibashi", 
	       "Fukui-Ishibashi",
	       "TOCA"
     	}; 

const char* traffic_nickname[] = { 
	       "nbna",  
	       "nasch", 
	       "munasch", 
	       "gipps", 
	       "mugipps", 
	       "vdr", 
	       "muvdr", 
	       "tsquare", 
	       "mutsquare", 
	       "fukui",
	       "mufukui",
	       "toca"
     	}; 
#endif

//===================================================================================
//  Channel Model                                                                                 
//-----------------------------------------------------------------------------------
DECLARE struct {
		int 	model; 
		int 	one_shot;  // a single broadcast propagation 
		int	markovian; // unless markovian, channel states are bernoulli iid 
		double  perror;
		double  transition; // transitional region
		struct  { double rate[2]; } gilbert;
	} channel;

#ifndef	 DECLARE_VAR
extern const char* hybrid_name[];
extern const char* hybrid_nickname[];
#else
const char* hybrid_name[] = { 
	       "D2F Function",  
	       "Constant Rebroadcast", 
	       "Inverse Probability",  
	       "D2F Function ONLY",  
	       "Constant Rebroadcast ONLY", 
	       "Inverse Probability ONLY",  
     	}; 
const char* hybrid_nickname[] = { 
	       "d2funcORED",  
	       "alwaysORED", 
	       "invrndORED",  
	       "d2funcONLY",  
	       "alwaysONLY", 
	       "invrndONLY",  
     	}; 
#endif

#ifndef	 DECLARE_VAR
extern const char* channel_name[];
extern const char* channel_nickname[];
#else
const char* channel_name[] = { 
	       "Ideal",  
	       "Bernoulli", 
	       "Transitional",  
	       "Gilber", 
	       "General"
     	}; 
const char* channel_nickname[] = { 
	       "idealCh",  
	       "bernoulliCh", 
	       "transCh",  
	       "gilberCh", 
	       "generalCh"
     	}; 
#endif

// Warning delivery service parameters
DECLARE char 	algostr[80];
DECLARE int     algo 		INIT_AS(ALGO_PFLOOD);

DECLARE double delay INIT_AS(0.0);  /*time elapsed between the accident and the warning reception*/
DECLARE double 	Pflood  	INIT_AS(1.0);   /*probability of fwd the warning*/

DECLARE double 	Pflood_constant;   /* copy of the above, to be used with d3flood */
DECLARE int    	d2flood_win;
DECLARE int    	d2flood_k;
DECLARE int    	d2flood_rule;
DECLARE int    	d3flood_h;
DECLARE int    	d3flood_rule;

DECLARE double 	HelloFreq;
DECLARE double 	txTimeHello;
DECLARE int    	MAX_CICLI_HELLO INIT_AS(1);
DECLARE double 	sizeHello       INIT_AS(20.0);
DECLARE int    	relay           INIT_AS(0);
DECLARE int    	Dfwd[1000];
DECLARE double 	epsilon;
DECLARE double 	txTime, 
		rxTime;
DECLARE double 	lastUpdate, 
		timeHello;
DECLARE int 	HelloCounter;
//shared parameters, whose meaning depends on the algorithmic context
DECLARE double par_x INIT_AS(0.0);
DECLARE int    par_y INIT_AS(0);


// Accident-related state variable
DECLARE long 	range;         /*transmission range*/
DECLARE double 	txRate;
DECLARE double 	min;          /*time interval after the accident */
DECLARE double 	AI;           /*time between two following broadcast cycles*/
DECLARE double 	timeAccident, 
		timeFirstAccident;

DECLARE int 	firstTx;      /*it is =1 if is the first transmission (all the cars around are not informed)*/


// end of forwarding: motivation 
DECLARE int 	endxnoone_further;
DECLARE int 	endxcoll;
DECLARE int 	endxnotx;

DECLARE int 	dup_relay;
DECLARE int 	info_relay;
DECLARE int 	collided;     /*number of cars not informed because of collision*/
/*number of cars not informed because back of 1-no one in range; 2- collision; 3-noForwarding; 4-other*/
DECLARE int 	tot_noinfo_backoorange,
		tot_noinfo_backcoll,
		tot_noinfo_notx,
		tot_noinfo_other;
		
DECLARE int 	carback;      /*number of cars back the last rx car -- potentially not informed*/
DECLARE int 	cars_accidentTime; /*total number of cars in the system at the accident*/


#define MAX_SIZE_BIG 50000
#define MAX_SIZE 10000
DECLARE int W[MAX_SIZE_BIG],C[MAX_SIZE],WC[MAX_SIZE][MAX_SIZE]; /*total number of cars receiving 1 or more Warnings, with 1 or more collisions*/
DECLARE int Wrel[MAX_SIZE]; /*total number of cars receiving 1 or more Warnings, with 1 or more collisions*/

DECLARE int 	totW;
DECLARE int 	informed;     /*number of informed cars*/
DECLARE double 	maxadvised;   /*max distance of the "last" informed car*/


/*----------------------------------------------
histogram
----------------------------------------------*/
DECLARE int have_1D 	INIT_AS(0);
DECLARE int have_09D 	INIT_AS(0);
DECLARE int have_05D 	INIT_AS(0);
DECLARE int have_025D 	INIT_AS(0);

DECLARE long NumW[MAX_T];
DECLARE struct stat fbuf;
DECLARE char postfix[1024];
DECLARE char prefix[1024];


DECLARE int 
    H_channel,
    H_numw, H_1D, H_09D, H_05D, H_025D, H_alfa, 
    H_dRelay, H_vicini, H_neighbors,
    H_speed,   H_headw, H_distance,
    H_velocity, H_flow, H_rho, H_density,
    H_ev,
//for estimating the error
    H_evA,    H_evB,    H_evC,    H_vAB,  H_vBC, H_vAC,
    H_exA,    H_exB,    H_exC,    H_xA_B,  H_xB_C, H_xA_C, H_exAB,  H_exBC,
    H_EM,     H_sRho,   H_nRho,   H_eRho,
    H_errVehicle, 	H_errFailing, 
    H_AerrVehicle,	H_AoverErrVehicle, 
    H_overErrVehicle, 	H_overErrFailing,
    H_failOvertake,  	H_succeedOvertake;

DECLARE double Hybrid INIT_AS(0.0);  // from 0.7x
DECLARE int AdaptiveMargin INIT_AS(0);
DECLARE int total_estimate INIT_AS(0), skip_estimate  INIT_AS(0);
DECLARE FILE *corrf INIT_AS(NULL);
DECLARE FILE *errf  INIT_AS(NULL);
#endif


